--[[
	CombatStats4 - Stat Damage Tracking
	AUTHOR: DmgInc on most forums / Glacier on official WoW forums
	THANKS TO:	The dude(ette) who made DPSPlus for showing me the light on how to parse the combat log
				The dude(ette) who made Clock for the time conversion routines
	
	Rev. 4.511	- Fixed healing record keeping in general.   CombatStats_IncrementSpecialDataRecord was one place
	01.18.2008	- revision done by Brian Beardmore ( http://www.realisticsoftware.com/pages/WoWAddons.html )
	
	Rev. 4.511	- Fixed bug to restore the reset behavior of clear the attack drop down
	11.23.07		menu also in CombatStats_Reset.
				- revision done by Brian Beardmore ( http://www.realisticsoftware.com/pages/WoWAddons.html )
	
	Rev. 4.510	- Update for WoW version 2.3
				- More memory usage improvements - (very very minor)
	11.13.07-BB	- revision done by Brian Beardmore ( http://www.realisticsoftware.com/pages/WoWAddons.html )

	Rev. 4.501	- Update for WoW version 2.2
	09.25.07-BB	- revision done by Brian Beardmore ( http://www.realisticsoftware.com/pages/WoWAddons.html )
	
	Rev. 4.50	- Added functionality to set the maximum updates-per-seconds.
	09.21.07	- Changed CombatStats update frequency so it is now caped at MaxUPS or current FPS
	(B.B.)		- Added functionality to position windows back to default. see CombatStats_ResetPos()
				- revision done by Brian Beardmore ( http://www.realisticsoftware.com/pages/WoWAddons.html )
	
	Rev. 4.492	- Fixed the call to CombatStats.Utils.NumToBool function.
	09.17.07-BB	- Fixed the FuBar menu gui configuration functionality.
				- Moved Fubar functionality to a separate file.
				- revision done by Brian Beardmore ( http://www.realisticsoftware.com/pages/WoWAddons.html )
	
	Rev. 4.491	- Fixed an initial state nil problem in importDataPre449 function.
	09.16.07-BB	- revision done by Brian Beardmore ( http://www.realisticsoftware.com/pages/WoWAddons.html )
	
	Rev. 4.49	- Memory usage improvements.
	09.15.07	- attackNames storage removed.
	(B.B.)		- specialAttackLog storage and usage removed.
				- split common lua code modules into separate support files.
				- revision done by Brian Beardmore ( http://www.realisticsoftware.com/pages/WoWAddons.html )
	
	Rev. 4.48	- Release the 4.47 beta changes
	07.27.07-BB	- revision done by Brian Beardmore ( http://www.realisticsoftware.com/pages/WoWAddons.html )
	
	Rev. 4.47	- Attempted a fix for "wowecon" exiting battle sutter
	07.18.07-BB	- Internal improvements for party battle data (end-of-fight)
	(beta)		- revision done by Brian Beardmore ( http://www.realisticsoftware.com/pages/WoWAddons.html )
				
	Rev. 4.46	- Removed Sea integration because only one simple utility function was being used and
	07.13.07		other newer addon have been released that seem to have replaced it.
	(B.B.)		- Unregistering events when they are not needed
				- Replaced Target_Frame function overrides with event
				- Reworked OnUpdate handler as documented in "HOWTO:_Use_OnUpdate_correct"
				- Lots of code cleanup
				- Improvements in attack event parsing, external addon integration, text localization
				- Reworked CombatStatsIB_OnLoad to clean it up and to have all the code in the lua file
				- A rough german translation for the new text additions to CS from version 3.81 on
				[ lots of time work put into looking for strange unexplain problems ]
					[ -- explained with WoW 2.1.3 patch 7/10 release NOTES ] 
					 * Improved the performance of conditional macros, slightly increasing the framerate.
					 * Fixed a crash triggered by many unit frame AddOns.
				- revision done by Brian Beardmore ( http://www.realisticsoftware.com/pages/WoWAddons.html )
	
	Rev. 4.45	- Attempted a fix for non english combat text parsing 
	07.05.07		[ Supporting CombatStats for non-english WoW version has turn out to be much more difficult then expected ]
	(beta)		- Improved main event handling - rewrote CombatStats_OnEvent
	(B.B.)		- Fixed the the blizzard globally defined attack text conversion for combat parsing
				- Fixed GUI attack menu so it stays sorted correctly
				- THIS BETA RELEASE HAS HAD MINIMUM TESTING AND NO TESTING ON NON-ENGLISH WOW

	Rev. 4.44	- Fixed "in combat" status with PLAYER_REGEN_ENABLED event handling & data start and stop 
	07.03.07		[ this fixes first strike combat damage and the combat timer start and stop ]
	(B.B.)			[ dynamic run-time DPS and end-of-fight data are now in agreement, except for known discrepancies ]
				- Improved CombatStats_OnUpdate functionality
				- Improved in combat data collection
				- Disabled the overridden ToggleDropDownMenu functionality
				- Fixed show and hide on target functionality (showonlywhentarget option)
				- Changed over to the blizzard globally defined attack text for combat parsing
					[ combat parsing in all languages should now be supported with this change ]

	Rev. 4.43	- Added FuBar support
	06.14.07	- Fixed a Titan Bar problem
	(B.B.)		- Fixed the regular DOT damages not being recorded correctly
				
	Rev. 4.42	- Lots of Titan integration improvements -- review the "CombatStatsTB_xxxx" functions
	06.08.07-BB	- Changed over to the blizzard globally defined time format strings in CombatStats_Clock_xx funcs
				
	Rev. 4.41	- Added the missing 'showttip' command
	06.03.07-BB	- Added the chat command support back for the old CS commands cmd "[on|off]"
				
	Rev. 4.40	- Update for WoW 2.1.0 interface
	05.23.07	(B.B.)
	
	Rev. 4.32	- Fixes to the InfoBar integration for tooltip and right click to display the detailed stats
	05.21.07	- Fixes in the Sea initialization function to handle incompatible Sea versions
	(B.B.)		- No longer save off obsolete old saved variables
	
	Rev. 4.31	- Changed the main default DPS output frame position to no longer depend on the player frame
	
	Rev. 4.3	- Reformatted the entire statics window to accommodate the high attack values
	05.04.07	- Added the addition drop down menu to allow removal of attack entries & reorder/sort attack menu
	(B.B.)		- Removed "Default" attack special ordering in the attack menu
				- Moved many many hard coded strings to the localization.lua file (non-english text variations are still needed)
				- Improved the attack grouping sub menu in the attack menu
				- Many internal code cleanup improvements
				- Added the InfoBar, Sea, Cosmos and Khaos CombatStats changes from http://www.petewood.info/Interface/AddOns/CombatStats/
				
	Rev. 4.01	- Fixed some nasty wackiness of the initial setting the group field data.
	04.13.07-BB	- Had to add functionality to verify and correct bad data entries.
				
	Rev. 4.00	- Lots of XML formatting cleanup in the detail frame window
	04.10.07-BB	- Changed attack dropdown menu to a hierarchical dropdown menu to
				  created groups of similar types and shorten the main menu count.
				- Fixed the date time stamp on events so they are now correct between system reboots.
				- revision done by Brian Beardmore ( http://www.realisticsoftware.com/pages/WoWAddons.html )
				
	Rev. 3.81	- Fixed the WoW v2.0 escape key usage of StaticPopup_EscapePressed
	12.28.06	- revision done by Brian Beardmore ( http://www.realisticsoftware.com/pages/WoWAddons.html )
	
	Rev. 3.8	- Fixed to support the WoW v2.0 changes
	12.19.06	- Removed the dependancy on CT_MasterMode because the module is now obsolete
	     		- Switched the right and left click behavior to be in line with other modes
	  			- revision done by Brian Beardmore ( http://www.realisticsoftware.com/pages/WoWAddons.html )
	  			
	Rev. 3.3	- ReEnabled the option to only show the stats on a mouseclick
	2.16.05		- DPS calculated is now 15 seconds w/no option to change
				- Added end of fight stats option to turn on/off
				- Slash commands will save between sessions when not using w/cosmos
	Rev. 3.2	- Added end of fight statistics
	2.09.05		- Using PLAYER_REGEN_DISABLED ENABLED to tell when you
				  are in/out of combat for end of fight info
	Rev. 3.1	- Lots of fixes to track pet stats
	2.07.05
	Rev. 3.0	- Initial version to track Defensive DPS
	12.xx.04
	Rev 2.03	- Now really really works w/o cosmos
	11.15.04
	Rev 2.02	- Now works w/o cosmos
	11.14.04	- Changed the text display frame
				- Fixed the crit % running out of the frame when it was 100%
				- Put "Default" attack at top of the list
				- Added in a "Total", put it at the bottom of the list
				- UI is now movable
				- UI will update itself while you are attacking if it is visible
				- Fixed a bug where the close "X" button couldn't be pressed
				- Now uses OnEvent and OnUpdate
				- Pet damage is now tracked
				- DOT damage totals are correctly shown
				- Hopefully got rid of bogus entries
	Rev 2.01	- First release w/new UI
	11.05.04
  ]]
 
--
-- addon constants (used by the internal code only)
--
kAddonName			= "CombatStats"
kAddonVersion		= "4.50"
kAddonVerNum		= 450
kAddonDateStr		= "2007.09.25"
	-- combat data storage types
kHitType 			= "hit"
kCritType 			= "crit"
kMissType 			= "miss"
kParryType 			= "parry"
kDodgeType 			= "dodge"
kDeflectType 		= "deflect"
kBlockType 			= "block"
kEvadeType 			= "evade"
kImmuneType 		= "immune"
kResistType 		= "resist"
kDotType			= "dot"
kHotDotType 		= "hot"
kYourType			= "Your"
	-- for storage of settings
kCSsetEnable = "OnOff"
kCSsetTarget = "HideOnNoTarget"
kCSsetEndofFight = "EndOfFight"
kCSsetMouseOver = "UseMouseOver"
kCSsetHide = "Hide"
kCSsetHideTTip = "HideTtip"
	-- for GUI format items
kNumberFormat = "%.1f"
kPercentFormat = "%.1f%%"
kPercentFormat2 = "%.2f%%"
kBracketPercentFormat = NORMAL_FONT_COLOR_CODE.."["..WHITE_FONT_COLOR_CODE..kPercentFormat..NORMAL_FONT_COLOR_CODE.."]"
kHitSwingFormat = WHITE_FONT_COLOR_CODE.."%d "..NORMAL_FONT_COLOR_CODE..CS_FRAME_HITS_TEXT..WHITE_FONT_COLOR_CODE.." %d "..NORMAL_FONT_COLOR_CODE..CS_FRAME_SWINGS_TEXT
kHitSwingFormatHEAL = WHITE_FONT_COLOR_CODE.."%d "..NORMAL_FONT_COLOR_CODE..CS_FRAME_HITS_TEXT_HEAL
kHitSwingFormatDOT = WHITE_FONT_COLOR_CODE.."%d "..NORMAL_FONT_COLOR_CODE..CS_FRAME_TICKS_TEXT
kHitSwingFormatDOT2 = WHITE_FONT_COLOR_CODE.."%d "..NORMAL_FONT_COLOR_CODE..CS_FRAME_HOT_TEXT
	-- for GUI default and empty values
kNumberTxt = format(kNumberFormat, 0)
kPercentTxt = format(kPercentFormat, 0)
kPercentTxt2 = format(kPercentFormat2, 0)
kBracketPercentTxt = format(kBracketPercentFormat, 0)
	-- garage collection type flags
kGCafterEOFType		= 1
kGCatRestType		= 2


-- Cleanup or fix when possible
kHealNmAType 		= "[Heal]"

--
--	Globals
--
CombatStatsTB_DPS_FREQUENCY = 0.1
CombatStatsTB_ID = kAddonName
CombatStats_DPSLen = 30;	-- Calculate DPS over the last 30 seconds

local partyFightDataOld = {}
local kDmgEndTotIdx = 1
local kDmgTakenEndTotIdx = 2
local kDmgTotIdx = 3
local kDmgTakenTotIdx = 4

local kTimeStampIdx = 1
local kDmgIdx = 2
local kPlayerIDIdx = 3



--[[
local gCSint_bat.playersTBL = {}
local gCSint_bat.playersDmgTots = { {0,0,0,0}, }		-- .022-.031 KB per loop
--local gCSint_bat.given = {}			-- .060-.069 KB per loop

-- .033sh when used
local gCSint_bat.timeStampTaken = {}
local gCSint_bat.dmgTaken = {}
local gCSint_bat.playerTaken = {}
local gCSint_bat.timeStampGiven = {}
local gCSint_bat.dmgGiven = {}
local gCSint_bat.playerGiven = {}
]]

--local gCSint_tmpStrArr = {}		-- .029-.059KB

local gCSint = {	
	--tmpStrArr = {},	-- temporary string array, used to avoid creating a bunch of local variables
	bat = {
		yourIdx = 1,				-- FIX THIS
		dpsTakenCalc = 0.0,
		dpsDoneCalc = 0.0,
		playersTBL = { },
		--playersDmgTots = {
		--	{ dmgEndTot = 0, dmgTakenEndTot = 0, dmgTot = 0, dmgTakenTot = 0 },			-- index 1 is for your information (see yourIdx)
		--},			-- .030-.035 KB per loop called
		playersDmgTots = { {0,0,0,0}, },		-- .029-.033 KB per loop called
		timeStampTaken = {},		-- battle timestamp array for when dmg was taken
		dmgTaken = {},				-- battle damages taken array
		playerTaken = {},			-- player idx in playersTBL that took the battle damages array
		takenCount = 0,				-- battle damages taken, timestamp, player arrays count
		startTakenIdx = 1, 			-- 
		
		timeStampGiven = {},		-- battle timestamp array for when dmg was taken
		dmgGiven = {},				-- battle damages taken array
		playerGiven = {},			-- player idx in playersTBL that took the battle damages array
		givenCount = 0,				-- battle damages taken, timestamp, player arrays count
		startGivenIdx = 1, 			-- 
		
	},
	
	bFirstTime = 1,
	bInCombat = 0,
	areActiveRegister = 0,
	
	dirtyAttackMenu = 0,
	timeSinceLastUpdate = 0,
	updateFreq = 0.15,

	-- last fight data
	lastFight = {
			start 		= 0,
			finish		= 0,
			playerDmg	= 0,
			petDmg		= 0,
		},
	-- other addon integration items
--	FuBarPlugin = nil,
--	FuBarTablet = nil,
	haveInfoBar = false,
	haveTitan = false,
	haveFuBar = false,
	haveKhaos = false,
}

-- Initialize these variables right away
_, _, _, gCSint.haveInfoBar, _, _, _ = GetAddOnInfo("InfoBar")
_, _, _, gCSint.haveTitan, _, _, _ = GetAddOnInfo("Titan")
_, _, _, gCSint.haveFuBar, _, _, _ = GetAddOnInfo("FuBar")
_, _, _, gCSint.haveKhaos, _, _, _ = GetAddOnInfo("Khaos")
gCSint.addonTitle = GetAddOnMetadata( kAddonName, "Title" )
gCSint.addonVersion = GetAddOnMetadata( kAddonName, "Version" )

--
--	 Saved and Restored Globals
--	
gCS = {}
--[[		see CombatStats_CheckLoadedVariables for internal variable initialization
	gCS.currentAttack		=	CS_TEXT_NA
	gCS.totalDamage 		=	0
	specialAttacks[CS_ATTACK_TOTAL_LABEL].totalHits 		=	0
	...
]]
	--	Configuration variable storage
CombatStats_Config = {}
--[[		see CombatStats_CheckLoadedVariables for internal variable initialization
	CombatStats_Config.CombatStats_OnOff = 1
	CombatStats_Config.CombatStats_HideOnNoTarget = 0
	CombatStats_Config.CombatStats_EndOfFight = 0
	CombatStats_Config.CombatStats_UseMouseOver = 0
	CombatStats_Config.CombatStats_Hide = 0
	CombatStats_Config.CombatStats_HideTtip = 0
	CombatStats_Config.CombatStats_AttackMenuSort = 0
	CombatStats_Config.CombatStats_AttackMenu1stGrping = 1
	CombatStats_Config.CombatStats_MaxUpPS = 30
]]

-------------------------------------------------------------------------------
-- Function Imports
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Function Prototypes
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Data Members
-------------------------------------------------------------------------------

local _L = {
	dpsForBg = "|cffff2020\%.1f|cffffd200\ / |cff20ff20\%.1f",			-- const
	dpsForSm = "|cffff2020\%.2f|cffffd200\ / |cff20ff20\%.2f",			-- const

};
-------------------------------------------------------------------------------
-- Constants
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Uses
-------------------------------------------------------------------------------
	
-------------------------------------------------------------------------------
-- Function Prototypes
-------------------------------------------------------------------------------

local CombatStats_IncrementSpecialDataRecord
local CombatStats_ShowLastFightDPS
local CombatStats_EnterCombatMode
local CombatStats_EndCombatMode
local CombatStats_AddDPSEntry
local CombatStats_AddDefDPSEntry
--
-- The Functional Code
--

function CombatStats_ChatCommandUsage(whichone)
	whichone = whichone or 3
	
	if ((whichone == 1) or (whichone == 3)) then
		cs_print(CS_CMD_HELP_TEXT)
		cs_print(CS_CMD_ENABLE_TEXT)
		cs_print(CS_CMD_DISABLE_TEXT)
		cs_print(CS_CMD_SHOWONTARGET_TEXT)
		cs_print(CS_CMD_SHOWALWAYS_TEXT)
		cs_print(CS_CMD_MOUSESTAT_TEXT)
		cs_print(CS_CMD_CLICKSTAT_TEXT)
		cs_print(CS_CMD_ENDFLIGHT_TEXT)
		cs_print(CS_CMD_NOENDFIGHT_TEXT)
		cs_print(CS_CMD_HIDE_TEXT)
		cs_print(CS_CMD_SHOW_TEXT)
		cs_print(CS_CMD_SHOWSTAT_TEXT)
		cs_print(CS_CMD_HIDETIP_TEXT)
		cs_print(CS_CMD_SHOWTIP_TEXT)
		cs_print(CS_CMD_RESET_TEXT)
		cs_print(CS_CMD_RESETWIN_TEXT)
		cs_print(CS_CMD_SETMAXUPS_TEXT)
	end
	if (whichone == 3) then
		cs_print("- - - - - -")
	end
	if ((whichone == 2) or (whichone == 3)) then
		cs_print(BLUE_FONT_COLOR_CODE..CS_CMD_HELP_OLD_TEXT)
		cs_print(BLUE_FONT_COLOR_CODE..CS_CMD_ENABLE_OLD_TEXT)
		cs_print(BLUE_FONT_COLOR_CODE..CS_CMD_TARGET_OLD_TEXT)
		cs_print(BLUE_FONT_COLOR_CODE..CS_CMD_MOUSEOVER_OLD_TEXT)
		cs_print(BLUE_FONT_COLOR_CODE..CS_CMD_ENDFLIGHT_OLD_TEXT)
		cs_print(BLUE_FONT_COLOR_CODE..CS_CMD_HIDE_OLD_TEXT)
		cs_print(BLUE_FONT_COLOR_CODE..CS_CMD_TOOLTIP_OLD_TEXT)
		cs_print(BLUE_FONT_COLOR_CODE..CS_CMD_RESET_OLD_TEXT)
	end
end

function CombatStats_ChatCommandUsageSummary()
	--cs_print(CS_CMD_HELP_SHORT_TEXT)
	--cs_print(CS_CMD_HELP_SHORT_OLD_TEXT)
	CombatStats_ChatCommandUsage()
end

function CombatStats_ChatCommandHandler(msg)
	--cs_print("CombatStats_ChatCommandHandler("..asText(msg)..")")
	
	if( not msg or msg == "" ) then
		CombatStats_ChatCommandUsageSummary()
	else
		local msg = string.lower(msg)
		local onoff = -1
		local firsti, lasti, command, setStr = string.find (msg, "(%w+) ([%w%.]+)")
		
		if (firsti ~= nil) then
			setStr = string.lower(setStr)
			if ((setStr == "yes") or (setStr == "y") or (setStr == "on")) then
				onoff = 1
			elseif ((setStr == "no") or (setStr == "n") or (setStr == "off")) then
				onoff = 0
			end
		else
			if(msg == CS_CMD_ENABLE) then
				command = CS_CMD_ENABLE onoff = 1
			elseif(msg == CS_CMD_DISABLE) then
				command = CS_CMD_ENABLE onoff = 0
			elseif(msg == CS_CMD_SHOWONTARGET) then
				command = CS_CMD_TARGET onoff = 1
			elseif(msg == CS_CMD_SHOWALWAYS) then
				command = CS_CMD_TARGET onoff = 0
			elseif(msg == CS_CMD_MOUSESTAT) then
				command = CS_CMD_MOUSEOVER onoff = 1
			elseif(msg == CS_CMD_CLICKSTAT) then
				command = CS_CMD_MOUSEOVER onoff = 0
			elseif(msg == CS_CMD_ENDFLIGHT) then
				command = CS_CMD_ENDFLIGHT onoff = 1
			elseif(msg == CS_CMD_NOENDFIGHT) then
				command = CS_CMD_ENDFLIGHT onoff = 0
			elseif(msg == CS_CMD_HIDE) then
				command = CS_CMD_HIDE onoff = 1
			elseif(msg == CS_CMD_SHOW) then
				command = CS_CMD_HIDE onoff = 0
			elseif(msg == CS_CMD_SHOWSTAT) then
				CombatStatsText_ShowFrame()
				return
			elseif(msg == CS_CMD_HIDETIP) then
				command = CS_CMD_TTIP onoff = 0
			elseif(msg == CS_CMD_SHOWTIP) then
				command = CS_CMD_TTIP onoff = 1
			elseif(msg == CS_CMD_RESET) then
				command = CS_CMD_RESET onoff = 1
			elseif(msg == CS_CMD_INFO_DETAILED1) then
				command = CS_CMD_INFO_DETAILED1 onoff = 1
			elseif(msg == CS_CMD_INFO_DETAILED2) then
				command = CS_CMD_INFO_DETAILED2 onoff = 1
			elseif(msg == CS_CMD_SETMAXUPS) then
				CombatStats_SetMaxUPS()			-- display the gui dialog because a value was not entered
				return
			elseif(msg == CS_CMD_RESETWIN) then
				CombatStats_ResetPos()
				return		
			end
		end
		
		if((command == CS_CMD_SETMAXUPS) and (setStr ~= nil)) then
			if (CombatStats.Dialogs.ChkSet_UpPSValue(setStr)) then
				cs_print("cs option "..command.." was set to "..CombatStats_Config.CombatStats_MaxUpPS)
			else
				cs_print("cs option "..command.." was NOT set")
			end
			return
		end

		if (onoff < 0) then
			CombatStats_ChatCommandUsageSummary()
			return
		elseif(command == CS_CMD_ENABLE) then
			CombatStats_SetEnable(onoff)
		elseif(command == CS_CMD_TARGET) then
			CombatStats_SetTarget(onoff)
		elseif(command == CS_CMD_MOUSEOVER) then
			CombatStats_SetMouseOver(onoff)
		elseif(command == CS_CMD_ENDFLIGHT) then
			CombatStats_SetEndFight(onoff)
		elseif(command == CS_CMD_HIDE) then
			CombatStats_SetHide(onoff)
		elseif(command == CS_CMD_TTIP) then
			if (onoff == 0) then
				CombatStats_SetHideTtip(1)
			else
				CombatStats_SetHideTtip(0)
			end
		elseif((command == CS_CMD_RESET) and (onoff == 1)) then
			CombatStats_Reset()
			CombatStats_UpdateVisibility()
			return
		elseif(command == CS_CMD_INFO_DETAILED1) then
			CombatStats_ChatCommandUsage(1)
			return
		elseif(command == CS_CMD_INFO_DETAILED2) then
			CombatStats_ChatCommandUsage(2)	
			return
		else
			CombatStats_ChatCommandUsageSummary()
			return
		end
		
		if (onoff == 0) then
			cs_print("cs option "..command.." is off")
		elseif (onoff == 1) then
			cs_print("cs option "..command.." is on")
		end
		
		CombatStats_UpdateVisibility()
	end
end

function CombatStats_UpdateVisibility(hasTarget)

	if (hasTarget == nil) then
		if (TargetFrame:IsVisible()) then
			hasTarget = ( UnitCanAttack("player", "target") or UnitCanAttack("target", "player") )
		else
			hasTarget = 0
		end
	end
	if (CombatStats_Config.CombatStats_OnOff == 1) then
		if (CombatStats_Config.CombatStats_HideOnNoTarget == 1) then
			if (hasTarget == 1) then
				if (CombatStats_Config.CombatStats_Hide == 0) then
					CombatStatsFrame:Show()
					CombatStats_UpdateDynCombatTexts()	-- the data shown may not be current, so update it
				end
			else
				CombatStatsFrame:Hide()
			end
		else
			if (CombatStats_Config.CombatStats_Hide == 0) then
				CombatStatsFrame:Show()
				CombatStats_UpdateDynCombatTexts()	-- the data shown may not be current, so update it
			else
				CombatStatsFrame:Hide()
			end
		end
	end
end

function CombatStats_Reset()
	gCSint.bat.takenCount = 0
	gCSint.bat.startTakenIdx = 1
	gCSint.bat.givenCount = 0
	gCSint.bat.startGivenIdx = 1
	-- and the arrays connected to them
	
	gCSint.bat.dpsTakenCalc = 0.0
	gCSint.bat.dpsDoneCalc = 0.0
	--gCSint.bat.playersDmgTots = {
	--	{ dmgEndTot = 0, dmgTakenEndTot = 0, dmgTot = 0, dmgTakenTot = 0 },			-- index 1 is for your information (see yourIdx)
	--}
	gCSint.bat.playersDmgTots = { {0,0,0,0}, }
	
	CombatStats.AttackHistDB.Clear();
	gCS.totalDamage = 0
	
	--specialAttacks[CS_ATTACK_TOTAL_LABEL] = CombatStats_New_SpecialAttack(CS_ATTACK_TOTAL_LABEL, 0)
	local attackRecord = CombatStats.AttackHistDB.GetAttackRec(CS_ATTACK_TOTAL_LABEL,true)
	
	gCS.currentAttack = CS_TEXT_NA
	CombatStatsText:SetText(CS_MINIFRAME_DEFAULT_TEXT)
	CombatStatsText_ResetFrameGUI()
end

local function CombatStats_ShowLastFightDPS()
	--
	--	Show the last fight DPS
	--
	local totalFightTime
	local playerDPS
	local petDPS
	local petPct
	local text = CS_LF_STATS_TEXT
	local playerN
	local data
	local i
	
	if( (gCSint.bInCombat == 0) and (gCSint.lastFight.finish > gCSint.lastFight.start)) then
		totalFightTime = gCSint.lastFight.finish - gCSint.lastFight.start
		petDPS = gCSint.lastFight.petDmg / totalFightTime
		playerDPS = gCSint.lastFight.playerDmg / totalFightTime
		
		if(lastFightPetDamge ~=0) then
			petPct =  (gCSint.lastFight.petDmg / gCSint.lastFight.playerDmg) * 100
		end
		text = text ..format(CS_LF_PLAYER_TEXT, totalFightTime,gCSint.lastFight.playerDmg,playerDPS)
		if(gCSint.lastFight.petDmg ~= 0) then
			text = text ..format(CS_LF_PET_TEXT,gCSint.lastFight.petDmg,petPct,petDPS)
		end
		cs_print(text)
		
		--[[
		-- does not work right now because only your damage is being recorded in gCSint.bat.playersDmgTots structure
		if (GetNumPartyMembers() > 0) then
			text = "  Party damage breakdown:\n"
			
			i = 0
			for playerN, data in pairs(gCSint.bat.playersDmgTots) do
				text = text .."   "..playerN..": "..data[kDmgTotIdx]
				text = text ..", "..format(CS_DYN_DPS_FORMAT, ((data[kDmgTotIdx] / gCSint.lastFight.playerDmg) * 100)).."%\n"
				i = 1+1
			end
			if (i>0) then cs_print(text) end
		end
		]]
		
		-- DEBUG
		--text = "gCSint.bat.playersDmgTots.[kDmgEndTotIdx]: "..gCSint.bat.playersDmgTots[kYourType][kDmgEndTotIdx]
		--text = "gCSint.bat.playersDmgTots.[kDmgTotIdx]: "..gCSint.bat.playersDmgTots[kYourType][kDmgTotIdx]
		--for w,y in pairs(battledmgGivenTBL) do
		--	text = text ..", t("..w.."): "..format("%.2f",(GetTime() - y.timestamp))
		--end
					
		--if(DEFAULT_CHAT_FRAME) then
		--	DEFAULT_CHAT_FRAME:AddMessage(text)
		--end
		--cs_print(text)
	end
end

local function CombatStats_EnterCombatMode()
	if (gCSint.bInCombat == 0) then
		gCSint.bInCombat = 1
		if ( CombatStats_Config.CombatStats_OnOff == 0 ) then
			return
		end
		
		gCSint.lastFight.start = GetTime()
		gCSint.lastFight.playerDmg = 0
		gCSint.lastFight.petDmg = 0
		
		gCSint.bat.givenCount = 0
		gCSint.bat.startGivenIdx = 1
		
		gCSint.bat.takenCount = 0
		gCSint.bat.startTakenIdx = 1
		
		for i=1, table.getn(gCSint.bat.playersDmgTots), 1 do
			gCSint.bat.playersDmgTots[i] = {0,0,0,0}
			--gCSint.bat.playersDmgTots[i] = { dmgEndTot = 0, dmgTakenEndTot = 0,
			--							dmgTot = 0, dmgTakenTot = 0 }
		end
		
		-- reset the update timer, but skip the first interations (to avoid displaying super high initial DPS values)
		gCSint.timeSinceLastUpdate = -(gCSint.updateFreq*1)
		
		CombatStats_UpdateVisibility()
		
	end
end

local function CombatStats_EndCombatMode()
	local i
	local v = {}
	
	if (gCSint.bInCombat == 1) then
		gCSint.bInCombat = 0
		if ( CombatStats_Config.CombatStats_OnOff == 0 ) then
			return
		end
		if (gCSint.lastFight.playerDmg > 0) then
			gCSint.lastFight.finish = GetTime()
			if(CombatStats_Config.CombatStats_EndOfFight  == 1) then
				CombatStats_ShowLastFightDPS()
			end
		end
		
		CombatStats_UpdateDynCombatInfo(GetTime())
		CombatStats_ChkGarbage(kGCafterEOFType)
		
	end
end


function CombatStats_OnEvent()
	local attackType = 0
	local stIdx = 0
	local endIdx = -1
	local sysdatetime = time()
	local tickCount = GetTime()
	
	--
	-- Event Handler
	-- the event switch order is based on most used and time critial
	--
	
	--if ((event == "PLAYER_REGEN_ENABLED") or (event == "PLAYER_LEAVE_COMBAT" )) then
	if (event == "PLAYER_REGEN_ENABLED") then
						-- note it appears PLAYER_LEAVE_COMBAT triggers after a single target falls
		-- switching from in combat to out of combat
		CombatStats_EndCombatMode()
		--gCSint.bInCombat = 0		-- always out of combat at this point
		return
		
	elseif ((event == "PLAYER_REGEN_DISABLED") or (event == "PLAYER_ENTER_COMBAT")) then
		CombatStats_EnterCombatMode()
		return
	
	--
	-- in combat events
	--
	elseif ( event == "CHAT_MSG_COMBAT_SELF_HITS"
		or event == "CHAT_MSG_COMBAT_SELF_MISSES"
		or event == "CHAT_MSG_SPELL_SELF_DAMAGE"
		or event == "CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE"
		or event == "CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE"
		or event == "CHAT_MSG_COMBAT_PET_HITS"
		or event == "CHAT_MSG_SPELL_PET_DAMAGE"
		or event == "CHAT_MSG_COMBAT_PET_MISSES"
		or event == "CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS"
		or event == "CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE"
		or event == "CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES"
		or event == "CHAT_MSG_SPELL_SELF_BUFF"
		or event == "CHAT_MSG_SPELL_PARTY_BUFF"
		or event == "CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF"
		) then
			--	Don't do anything else at this point unless
			--	combat stats is turned on (should never have to use this check if if register and unregister code is working)
			--
			if ( CombatStats_Config.CombatStats_OnOff == 0 ) then
				return
			end
			-- see if we can match the attack text
			attackType = CombatStats.ComParse.ParseCombatEvent(event, arg1)
			-- make sure we are set in combat for these events
			if ( event == "CHAT_MSG_COMBAT_PET_MISSES" 
				or event == "CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS"
				) then
					CombatStats_EnterCombatMode()
				end
	
	--
	-- System type events
	--
	elseif ((event == "PLAYER_FOCUS_CHANGED") or (event == "PLAYER_TARGET_CHANGED")) then
		-- replacement event for the function TargetFrame_OnShow, TargetFrame_OnHide overrides
		-- the newer PLAYER_FOCUS_CHANGED event should covers PLAYER_TARGET_CHANGED - but it does not seem to
		CombatStats_UpdateVisibility()
		return
		
	elseif (event == "VARIABLES_LOADED") then
		CombatStats.AttackHistDB.Load()
		
		CombatStats_CheckLoadedVariables()
		CombatStats.DetailFrame.SetDirtyAttackMenu()
		CombatStats_UpdateWhenActiveRegistery()
		
		CombatStats_UpdateVisibility()
		
		-- no longer need to track this event
		this:UnregisterEvent("VARIABLES_LOADED")
		return
	
	elseif (event == "PLAYER_UPDATE_RESTING") then
		CombatStats_ChkGarbage(kGCatRestType)
		return
		
	else
		return
	end
	
	--
	--	Don't do anything else at this point unless
	--	combat stats is turned on (should never have to use this check if if register and unregister code is working)
	--
--	if ( CombatStats_Config.CombatStats_OnOff == 0 ) then
--		return
--	end

	-- no reason to continue if we do not have an attack type
	if (attackType == 0) then return end
	
	-- "CHAT_MSG_COMBAT_SELF_HITS", "CHAT_MSG_COMBAT_SELF_MISSES", "CHAT_MSG_SPELL_SELF_DAMAGE"
	-- "CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE", "CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE"
	-- 	SECTION
	if (attackType == 1) then
		CombatStats_EnterCombatMode()
		CombatStats_AddSpecialData(CombatStats.ComParse.a.spell,kDotType,CombatStats.ComParse.a.damage,sysdatetime)
		CombatStats_AddDPSEntry(tickCount, kYourType, CombatStats.ComParse.a.damage)
		
	elseif (attackType == 2) then
		CombatStats_EnterCombatMode()
		CombatStats_AddSpecialData(CombatStats.ComParse.a.spell,kHitType,CombatStats.ComParse.a.damage,sysdatetime)
		CombatStats_AddDPSEntry(tickCount, kYourType, CombatStats.ComParse.a.damage)
		
	elseif (attackType == 3) then
		CombatStats_EnterCombatMode()
		CombatStats_AddSpecialData(CombatStats.ComParse.a.spell,kHitType,CombatStats.ComParse.a.damage,sysdatetime)
		CombatStats_AddDPSEntry(tickCount, kYourType, CombatStats.ComParse.a.damage)
		
	elseif (attackType == 4) then
		CombatStats_EnterCombatMode()
		CombatStats_AddSpecialData(CS_ATTACK_GEN_DEF_LABEL,kHitType,CombatStats.ComParse.a.damage,sysdatetime)
		CombatStats_AddDPSEntry(tickCount, kYourType, CombatStats.ComParse.a.damage)
		
	elseif (attackType == 5) then
		CombatStats_EnterCombatMode()
		CombatStats_AddSpecialData(CombatStats.ComParse.a.spell,kCritType,CombatStats.ComParse.a.damage,sysdatetime)
		CombatStats_AddDPSEntry(tickCount, kYourType, CombatStats.ComParse.a.damage)
		
	elseif (attackType == 6) then
		CombatStats_EnterCombatMode()
		CombatStats_AddSpecialData(CS_ATTACK_GEN_DEF_LABEL,kCritType,CombatStats.ComParse.a.damage,sysdatetime)
		CombatStats_AddDPSEntry(tickCount, kYourType, CombatStats.ComParse.a.damage)
		
	elseif (attackType == 7) then
		CombatStats_AddSpecialData(CS_ATTACK_GEN_DEF_LABEL,kMissType,0,sysdatetime)
		
	elseif (attackType == 8) then
		CombatStats_AddSpecialData(CS_ATTACK_GEN_DEF_LABEL,kParryType,0,sysdatetime)
		
	elseif (attackType == 9) then
		CombatStats_AddSpecialData(CS_ATTACK_GEN_DEF_LABEL,kEvadeType,0,sysdatetime)
		
	elseif (attackType == 10) then
		CombatStats_AddSpecialData(CS_ATTACK_GEN_DEF_LABEL,kDodgeType,0,sysdatetime)
		
	elseif (attackType == 11) then
		CombatStats_AddSpecialData(CS_ATTACK_GEN_DEF_LABEL,kDeflectType,0,sysdatetime)
		
	elseif (attackType == 12) then
		CombatStats_AddSpecialData(CS_ATTACK_GEN_DEF_LABEL,kBlockType,0,sysdatetime)
		
	elseif (attackType == 13) then
		CombatStats_AddSpecialData(CombatStats.ComParse.a.spell,kBlockType,0,sysdatetime)
		
	elseif (attackType == 14) then
		CombatStats_AddSpecialData(CombatStats.ComParse.a.spell,kDeflectType,0,sysdatetime)
		
	elseif (attackType == 15) then
		CombatStats_AddSpecialData(CombatStats.ComParse.a.spell,kDodgeType,0,sysdatetime)
		
	elseif (attackType == 16) then
		CombatStats_AddSpecialData(CombatStats.ComParse.a.spell,kEvadeType,0,sysdatetime)
		
	elseif (attackType == 17) then
		CombatStats_AddSpecialData(CombatStats.ComParse.a.spell,kParryType,0,sysdatetime)
		
	elseif (attackType == 18) then
		CombatStats_AddSpecialData(CombatStats.ComParse.a.spell,kResistType,0,sysdatetime)
		
	elseif (attackType == 19) then
		CombatStats_AddSpecialData(CombatStats.ComParse.a.spell,kImmuneType,0,sysdatetime)
		
	elseif (attackType == 20) then
		CombatStats_AddSpecialData(CombatStats.ComParse.a.spell,kMissType,0,sysdatetime)
		
	-- "CHAT_MSG_COMBAT_PET_HITS" SECTION
	elseif (attackType == 21) then
		CombatStats_AddSpecialData(CS_ATTACK_PET_DEF_LABEL,kCritType,CombatStats.ComParse.a.damage,sysdatetime)
		CombatStats_AddDPSEntry(tickCount, kYourType, CombatStats.ComParse.a.damage)
		
	elseif (attackType == 22) then
		CombatStats_AddSpecialData(CS_ATTACK_PET_DEF_LABEL,kHitType,CombatStats.ComParse.a.damage,sysdatetime)
		CombatStats_AddDPSEntry(tickCount, kYourType, CombatStats.ComParse.a.damage)
	
	-- "CHAT_MSG_SPELL_PET_DAMAGE" SECTION
	elseif (attackType == 23) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_PRE_LABEL..CombatStats.ComParse.a.spell,kHitType,CombatStats.ComParse.a.damage,sysdatetime)
			CombatStats_AddDPSEntry(tickCount, kYourType, CombatStats.ComParse.a.damage)
		end
		
	elseif (attackType == 24) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_PRE_LABEL..CombatStats.ComParse.a.spell,kCritType,CombatStats.ComParse.a.damage,sysdatetime)
			CombatStats_AddDPSEntry(tickCount, kYourType, CombatStats.ComParse.a.damage)
		end
		
	elseif (attackType == 25) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_PRE_LABEL..CombatStats.ComParse.a.spell,kBlockType,0,sysdatetime)
		end
		
	elseif (attackType == 26) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_PRE_LABEL..CombatStats.ComParse.a.spell,kDodgeType,0,sysdatetime)
		end
		
	elseif (attackType == 27) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_PRE_LABEL..CombatStats.ComParse.a.spell,kEvadeType,0,sysdatetime)
		end
		
	elseif (attackType == 28) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_PRE_LABEL..CombatStats.ComParse.a.spell,kImmuneType,0,sysdatetime)
		end
		
	elseif (attackType == 29) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_PRE_LABEL..CombatStats.ComParse.a.spell,kResistType,0,sysdatetime)
		end
		
	elseif (attackType == 30) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_PRE_LABEL..CombatStats.ComParse.a.spell,kMissType,0,sysdatetime)
		end
		
	elseif (attackType == 31) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_PRE_LABEL..CombatStats.ComParse.a.spell,kMissType,0,sysdatetime)
		end
		
	elseif (attackType == 32) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_PRE_LABEL..CombatStats.ComParse.a.spell,kImmuneType,0,sysdatetime)
		end
		
	elseif (attackType == 33) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_PRE_LABEL..CombatStats.ComParse.a.spell,kMissType,0,sysdatetime)
		end
		
	elseif (attackType == 34) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_PRE_LABEL..CombatStats.ComParse.a.spell,kDeflectType,0,sysdatetime)
		end
	
	-- "CHAT_MSG_COMBAT_PET_MISSES" SECTION
	elseif (attackType == 35) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_DEF_LABEL,kMissType,0,sysdatetime)
		end
		
	elseif (attackType == 36) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_DEF_LABEL,kParryType,0,sysdatetime)
		end
		
	elseif (attackType == 37) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_DEF_LABEL,kEvadeType,0,sysdatetime)
		end
		
	elseif (attackType == 38) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_DEF_LABEL,kDodgeType,0,sysdatetime)
		end
		
	elseif (attackType == 39) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_DEF_LABEL,kDeflectType,0,sysdatetime)
		end
		
	elseif (attackType == 40) then
		if (CombatStats.ComParse.a.petName == UnitName('pet') or CombatStats.ComParse.a.petName == 'your pet') then
			CombatStats_AddSpecialData(CS_ATTACK_PET_DEF_LABEL,kBlockType,0,sysdatetime)
		end
		
	-- "CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS" SECTION
	elseif (attackType == 41) then
		CombatStats_AddDefDPSEntry(tickCount, kYourType,CombatStats.ComParse.a.damage)
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kHitType,CombatStats.ComParse.a.damage,sysdatetime)
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kBlockType,0,sysdatetime)
		
	elseif (attackType == 42) then
		CombatStats_AddDefDPSEntry(tickCount, kYourType,CombatStats.ComParse.a.damage)
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kHitType,CombatStats.ComParse.a.damage,sysdatetime)
		
	elseif (attackType == 43) then
		CombatStats_AddDefDPSEntry(tickCount, kYourType,CombatStats.ComParse.a.damage)
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kCritType,CombatStats.ComParse.a.damage,sysdatetime)
		
	-- "CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE" SECTION
	elseif (attackType == 44) then
		CombatStats_AddDefDPSEntry(tickCount, kYourType,CombatStats.ComParse.a.damage)
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kHitType,CombatStats.ComParse.a.damage,sysdatetime)
		
	elseif (attackType == 45) then
		CombatStats_AddDefDPSEntry(tickCount, kYourType,CombatStats.ComParse.a.damage)
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kCritType,CombatStats.ComParse.a.damage,sysdatetime)
		
	elseif (attackType == 46) then
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kBlockType,0,sysdatetime)
		
	elseif (attackType == 47) then
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kDeflectType,0,sysdatetime)
		
	elseif (attackType == 48) then
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kDodgeType,0,sysdatetime)
		
	elseif (attackType == 49) then
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kEvadeType,0,sysdatetime)
		
	elseif (attackType == 50) then
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kImmuneType,0,sysdatetime)
		
	elseif (attackType == 51) then
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kMissType,0,sysdatetime)
		
	elseif (attackType == 52) then
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kParryType,0,sysdatetime)
		
	elseif (attackType == 53) then
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kResistType,0,sysdatetime)
		
	-- "CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES" SECTION	
	elseif (attackType == 54) then
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kMissType,0,sysdatetime)
		
	elseif (attackType == 55) then
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kParryType,0,sysdatetime)
		
	elseif (attackType == 56) then
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kEvadeType,0,sysdatetime)
		
	elseif (attackType == 57) then
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kDodgeType,0,sysdatetime)
		
	elseif (attackType == 58) then
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kDeflectType,0,sysdatetime)
		
	elseif (attackType == 59) then
		CombatStats_AddSpecialData(CS_ATTACK_DEFENSE_LABEL,kBlockType,0,sysdatetime)
	
	-- "CHAT_MSG_SPELL_SELF_BUFF", "CHAT_MSG_SPELL_PARTY_BUFF", "CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF" SECTION	
	elseif (attackType == 60) then
		if ( CombatStats.ComParse.a.spell and CombatStats.ComParse.a.healP ) then
			CombatStats_AddSpecialData(CS_ATTACK_HEAL_TOT_LABEL,kCritType,CombatStats.ComParse.a.healP,sysdatetime, 1)
			CombatStats_AddSpecialData(CS_ATTACK_HEAL_PRE_LABEL..CombatStats.ComParse.a.spell,kCritType,CombatStats.ComParse.a.healP,sysdatetime, 1)
			
		end
	elseif (attackType == 61) then
		if ( CombatStats.ComParse.a.spell and CombatStats.ComParse.a.healP ) then
			CombatStats_AddSpecialData(CS_ATTACK_HEAL_TOT_LABEL,kHitType,CombatStats.ComParse.a.healP,sysdatetime, 1)
			CombatStats_AddSpecialData(CS_ATTACK_HEAL_PRE_LABEL..CombatStats.ComParse.a.spell,kHitType,CombatStats.ComParse.a.healP,sysdatetime, 1)
		end
	
	-- "CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS", "CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS" SECTION	
	elseif (attackType == 62) then
		if ( CombatStats.ComParse.a.spell and CombatStats.ComParse.a.healP ) then
			CombatStats_AddSpecialData(CS_ATTACK_HEAL_TOT_LABEL,kHitType,CombatStats.ComParse.a.healP,sysdatetime, 1)
			CombatStats_AddSpecialData(CS_ATTACK_HEAL_PRE_LABEL..CombatStats.ComParse.a.spell,kHotDotType,CombatStats.ComParse.a.healP,sysdatetime, 1)
		end
	
	-- "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS" SECTION	
	elseif (attackType == 63) then
		if ( not string.find(arg1, "'s") and CombatStats.ComParse.a.spell and CombatStats.ComParse.a.healP ) then
			CombatStats_AddSpecialData(CS_ATTACK_HEAL_TOT_LABEL,kHitType,CombatStats.ComParse.a.healP,sysdatetime, 1)
			CombatStats_AddSpecialData(CS_ATTACK_HEAL_PRE_LABEL..CombatStats.ComParse.a.spell,kHotDotType,CombatStats.ComParse.a.healP,sysdatetime, 1)
		end
	end
	
end


--
--	Add/Check for attack(s)
--
	
function CombatStats_AddSpecialData(specialName,type,dmg,sysdatetime,heal)
	--cs_print("Adding : "..specialName .." with damage : "..dmg..", at : "..sysdatetime)
	local	tmpType
	
	--
	--	If we haven't logged this attack
	--	Create a record in the table for it
	--

	-- make sure the dmg variable is a number 
	--		- already stored variables have been corrected in the load verify function
	dmg = tonumber(dmg)
	
	if(specialName ~= CS_ATTACK_DEFENSE_LABEL and not heal ) then
		gCSint.lastFight.playerDmg = gCSint.lastFight.playerDmg + dmg
		gCS.totalDamage = gCS.totalDamage + dmg
	end
	
	for creatureName,tempJunk in string.gmatch(specialName, CS_PET_ATTACK_CHK) do
		--cs_print("creatureName : "..creatureName)
		--cs_print("tempJunk : "..tempJunk)
		--cs_print("Adding pet special : "..specialName.. " with damage of : "..dmg)
		gCSint.lastFight.petDmg = gCSint.lastFight.petDmg + dmg
	end
	
	if (type == kDotType) then
		specialName = specialName ..CS_ATTACK_DOT_SUF_LABEL
	elseif ( type == kHotDotType ) then
		specialName = specialName ..CS_ATTACK_HOT_SUF_LABEL
	end
	
	-- TOTAL HACK HERE ---- FIX
	if (CombatStats.AttackHistDB.GetAttackRec(CS_ATTACK_TOTAL_LABEL,false) == nil) then
		CombatStats.AttackHistDB.GetAttackRec(CS_ATTACK_TOTAL_LABEL,true,true)
	end
	
	if (CombatStats.AttackHistDB.GetAttackRec(specialName,false) == nil) then
		CombatStats.AttackHistDB.GetAttackRec(specialName,true,true)
		CombatStats.DetailFrame.SetDirtyAttackMenu()
	end
	
	--
	--	Now add the data
	--
	
	CombatStats_IncrementSpecialDataRecord(specialName,type,dmg,sysdatetime,heal)
	
	--	Defensive info
	--	Don't add to overall stats
	if ((type == kDotType) or (type == kHotDotType)) then tmpType = kHitType else tmpType = type end
	CombatStats_IncrementSpecialDataRecord(CS_ATTACK_TOTAL_LABEL,tmpType,dmg,sysdatetime,(specialName == CS_ATTACK_DEFENSE_LABEL or heal))
	
	
	if ( gCS.currentAttack and gCS.currentAttack == specialName ) then
		CombatStats_UpdateDetails(specialName)
	end
end

function CombatStats_IncrementSpecialDataRecord(recordName,type,dmg,sysdatetime,heal)
--	local specAttck = specialAttacks[recordName]
	local specAttck = CombatStats.AttackHistDB.GetAttackRec(recordName,false)
	
	if (specAttck == nil) then cs_print("GetAttackRec returned nil for "..recordName); return; end
	
	-- make sure the dmg variable is a number - already stored variable have been corrected in the load verify function
	dmg = tonumber(dmg)
	
	--
	--	Now update the records data if healing flag is not set
	--
	
--	if (not heal) then
		specAttck.totalSwings = specAttck.totalSwings + 1
		specAttck.lastUpdate = sysdatetime
		--gCSint.dirtyAttackMenu = 1
		CombatStats.DetailFrame.SetDirtyAttackMenu()
		if(type == kCritType) then
			specAttck.isDOT = 0
			specAttck.totalCrits = specAttck.totalCrits +1
			specAttck.totalCritDmg = specAttck.totalCritDmg + dmg
			specAttck.totalHits = specAttck.totalHits + 1
			specAttck.lastCrit = sysdatetime
			--
			--	Check to see if this dmg
			--	is a new min or max
			--
			if((dmg < specAttck.minCrit) or (specAttck.minCrit == 0)) then
				specAttck.minCrit = dmg
				specAttck.minCritTime = sysdatetime
			end
			if(dmg > specAttck.maxCrit) then
				specAttck.maxCrit = dmg
				specAttck.maxCritTime = sysdatetime
			end
		elseif(type == kHitType) then
			specAttck.isDOT = 0
			specAttck.totalNonCrits = specAttck.totalNonCrits +1
			specAttck.totalRegDmg = specAttck.totalRegDmg + dmg
			specAttck.totalHits = specAttck.totalHits + 1
			--
			--	Check to see if this dmg
			--	is a new min or max
			--
			if((dmg < specAttck.minReg) or (specAttck.minReg == 0)) then
				specAttck.minReg = dmg
				specAttck.minRegTime = sysdatetime
			end
			if(dmg > specAttck.maxReg) then
				specAttck.maxReg = dmg
				specAttck.maxRegTime = sysdatetime
			end
		elseif(type == kDotType) then
			specAttck.isDOT = 1
			specAttck.dotTicks = specAttck.dotTicks +1
			specAttck.dotDmg = specAttck.dotDmg + dmg
			specAttck.totalHits = specAttck.totalHits + 1
			--
			--	Check to see if this dmg
			--	is a new min or max
			--
			if((dmg < specAttck.minReg) or (specAttck.minReg == 0)) then
				specAttck.minReg = dmg
				specAttck.minRegTime = sysdatetime
			end
			if(dmg > specAttck.maxReg) then
				specAttck.maxReg = dmg
				specAttck.maxRegTime = sysdatetime
			end
		elseif(type == kHotDotType) then
			specAttck.isDOT = 2
			specAttck.dotTicks = specAttck.dotTicks +1
			specAttck.dotDmg = specAttck.dotDmg + dmg
			specAttck.totalHits = specAttck.totalHits + 1
			--
			--	Check to see if this dmg
			--	is a new min or max
			--
			if((dmg < specAttck.minReg) or (specAttck.minReg == 0)) then
				specAttck.minReg = dmg
				specAttck.minRegTime = sysdatetime
			end
			if(dmg > specAttck.maxReg) then
				specAttck.maxReg = dmg
				specAttck.maxRegTime = sysdatetime
			end
		elseif(type == kMissType) then
			specAttck.totalMisses = specAttck.totalMisses + 1
		elseif(type == kDodgeType) then
			specAttck.totalDodged = specAttck.totalDodged + 1
		elseif(type == kParryType) then
			specAttck.totalParried = specAttck.totalParried + 1
		elseif(type == kBlockType) then
			specAttck.totalBlocked = specAttck.totalBlocked + 1
		elseif(type == kEvadeType) then
			specAttck.totalEvaded = specAttck.totalEvaded + 1
		elseif(type == kResistType) then
			specAttck.totalResisted = specAttck.totalResisted + 1
		elseif(type == kImmuneType) then
			specAttck.totalImmuned = specAttck.totalImmuned + 1
		elseif(type == kDeflectType) then
			specAttck.totalDeflected = specAttck.totalDeflected + 1
		end
		
--	end
	
	
end

function CombatStats_SubtractFromSpecialDataTotals(attackRecord)
	-- passing in the attackRecord a slower operation than individual args 
	-- so this function should be a low use call
	
	--local specAttck = specialAttacks[CS_ATTACK_TOTAL_LABEL]
	local specAttck = CombatStats.AttackHistDB.GetAttackRec(CS_ATTACK_TOTAL_LABEL,false)
	
	specAttck.dotDmg = specAttck.dotDmg - attackRecord.dotDmg
	specAttck.dotTicks = specAttck.dotTicks - attackRecord.dotTicks
	specAttck.totalSwings = specAttck.totalSwings - attackRecord.totalSwings
	specAttck.totalMisses = specAttck.totalMisses - attackRecord.totalMisses
	specAttck.totalDodged = specAttck.totalDodged - attackRecord.totalDodged
	specAttck.totalParried = specAttck.totalParried - attackRecord.totalParried
	specAttck.totalEvaded = specAttck.totalEvaded - attackRecord.totalEvaded
	specAttck.totalBlocked = specAttck.totalBlocked - attackRecord.totalBlocked
	specAttck.totalResisted = specAttck.totalResisted - attackRecord.totalResisted
	specAttck.totalImmuned = specAttck.totalImmuned - attackRecord.totalImmuned
	specAttck.totalDeflected = specAttck.totalDeflected - attackRecord.totalDeflected
	specAttck.totalHits = specAttck.totalHits - attackRecord.totalHits
	specAttck.totalNonCrits = specAttck.totalNonCrits - attackRecord.totalNonCrits
	specAttck.totalCrits = specAttck.totalCrits - attackRecord.totalCrits
	specAttck.totalRegDmg = specAttck.totalRegDmg - attackRecord.totalRegDmg
	specAttck.totalCritDmg = specAttck.totalCritDmg - attackRecord.totalCritDmg
end

--
-- Update Handler
--

function CombatStats_OnUpdate(elapsed)
	gCSint.timeSinceLastUpdate = gCSint.timeSinceLastUpdate + elapsed
	
	if (gCSint.bInCombat == 1) then
		local curFPS
		-- Check to see if its time to update
		if (gCSint.timeSinceLastUpdate > gCSint.updateFreq) then
			CombatStats_UpdateDynCombatInfo(GetTime())
			gCSint.timeSinceLastUpdate = 0
			
			curFPS = GetFramerate()
			if (curFPS > CombatStats_Config.CombatStats_MaxUpPS) then curFPS = CombatStats_Config.CombatStats_MaxUpPS end
			if (curFPS < 1) then curFPS = 1 end
			gCSint.updateFreq = 1/curFPS	-- dynamically change the updates frequency depending on the FPS available
		end
	end
end

local function CombatStats_SetDynCombatTexts(theString)
	-- cs_print("CombatStats_SetDynCombatTexts called with: "..theString)		-- DEBUG
	
	if(CombatStatsFrame:IsVisible()) then
		CombatStatsText:SetText(theString)
	end
	-- Update InfoBar if it is being used
	if(InfoBarFrame ~= nil) then
		if(CombatStatsIBFrame:IsVisible()) then
			CombatStatsIBFrameCenteredText:SetText(theString)
		end
	end
	-- Update FuBar if it is being used
	CombatStats.FuBar.Update_BarText(theString)
	
	-- FIX --> somehow set CombatStatsTB_GetButtonText()
end

--[[ 2007.08.16
	After a bunch of testing, here is what the memory it doing (note garage collections will recollect most or all this memory at some time)
	[ using wow 2.1.3  along with UpdateAddOnMemoryUsage() local memBefore = GetAddOnMemoryUsage(kAddonName) ]
	[ Note there appears to be no overhead memory used by UpdateAddOnMemoryUsage & GetAddOnMemoryUsage
	 * 0.097 KB per call --> format("%.2f/%.2f", gCSint.bat.dpsTakenCalc, gCSint.bat.dpsDoneCalc)
	 * 0.063 KB per call --> format("|cffff2020\%.2f|cffffd200\ / |cff20ff20\%.2f", gCSint.bat.dpsTakenCalc, gCSint.bat.dpsDoneCalc)
	 * 0.066 KB per call --> local theString = format("|cffff2020\%.2f|cffffd200\ / |cff20ff20\%.2f", gCSint.bat.dpsTakenCalc, gCSint.bat.dpsDoneCalc)
	 * 0.116 KB per call --> local theString = TEXT(gCSint.bat.dpsTakenCalc).." / "..TEXT(gCSint.bat.dpsDoneCalc)
	 * 0.055 to 0.060 KB per call --> local theString = format("|cffff2020\%.1f|cffffd200\ / |cff20ff20\%.1f", gCSint.bat.dpsTakenCalc, gCSint.bat.dpsDoneCalc)
	 * 0.061 KB per call --> gCSint.bat.theString = format("|cffff2020\%.2f|cffffd200\ / |cff20ff20\%.2f", gCSint.bat.dpsTakenCalc, gCSint.bat.dpsDoneCalc)
	 * 0.112 KB per call --> local theString = ( RED_FONT_COLOR_CODE.. format(TEXT(CS_DYN_DPS_FORMAT), (gCSint.bat.dpsTakenCalc )) ..NORMAL_FONT_COLOR_CODE.. 
	 	" / " ..GREEN_FONT_COLOR_CODE.. format(TEXT(CS_DYN_DPS_FORMAT), gCSint.bat.dpsDoneCalc ))
	 with wow 2.2
	  * 0.079 KB per call --> local theString = "|cffff2020"..floor(gCSint.bat.dpsTakenCalc).."|cffffd200\ / |cff20ff20"..floor(gCSint.bat.dpsDoneCalc) 
	  * 0.052 KB per call --> local theString = format("|cffff2020\%d|cffffd200\ / |cff20ff20\%d", gCSint.bat.dpsTakenCalc, gCSint.bat.dpsDoneCalc)
	  * 0.053 KB per call --> local theString = format(forA, gCSint.bat.dpsTakenCalc, gCSint.bat.dpsDoneCalc)
	  * 0.059 KB per call --> local theString = format("|cffff2020\%.1f|cffffd200\ / |cff20ff20\%.1f", gCSint.bat.dpsTakenCalc, gCSint.bat.dpsDoneCalc)
]]
function CombatStats_UpdateDynCombatTexts()
	-- cs_print("CombatStats_UpdateDynCombatTexts")			-- DEBUG
	local theString
	
	if (gCSint.bat.dpsDoneCalc ~= 0 or gCSint.bat.dpsTakenCalc ~= 0) then
		if (gCSint.bat.dpsDoneCalc > 10) then
			--theString = format("|cffff2020\%.1f|cffffd200\ / |cff20ff20\%.1f", gCSint.bat.dpsTakenCalc, gCSint.bat.dpsDoneCalc)
			theString = format(_L.dpsForBg, gCSint.bat.dpsTakenCalc, gCSint.bat.dpsDoneCalc)
		else
			-- only a savings of about 0.005 KB
			--theString = format("|cffff2020\%.2f|cffffd200\ / |cff20ff20\%.2f", gCSint.bat.dpsTakenCalc, gCSint.bat.dpsDoneCalc)
			theString = format(_L.dpsForSm, gCSint.bat.dpsTakenCalc, gCSint.bat.dpsDoneCalc)
		end
	else
		theString = CS_MINIFRAME_DEFAULT_TEXT
	end
	
	-- Update CS main mini window
	if(CombatStatsFrame:IsVisible()) then
		CombatStatsText:SetText( theString )
	end
	-- Update InfoBar if it is being used
	if(InfoBarFrame ~= nil) then
		if(CombatStatsIBFrame:IsVisible()) then
			CombatStatsIBFrameCenteredText:SetText(theString)
		end
	end
	-- Update FuBar if it is being used
	CombatStats.FuBar.Update_BarText(theString)
	
	-- ### KEEP CombatStatsTB_GetButtonText() function in sync with this function
end

function CombatStats_UpdateDynCombatInfo(curtime)
	local oldesttime = 0
	local oldesttimeTaken = 0
	local deleteindex = 0
	local deleteindexTaken = 0
		
	--
	--	Offensive
	--
	for i = gCSint.bat.startGivenIdx, gCSint.bat.givenCount do
--	for k,v in pairs(battledmgGivenTBL) do
		-- Get rid of old dps entries and adjust the totals
		if ( (curtime - CombatStats_DPSLen) > gCSint.bat.timeStampGiven[i] ) then
			--cs_print("Offensive subtract: "..i)
			--cs_print("gCSint.bat.playerGiven[i]: "..gCSint.bat.playerGiven[i])
			
			-- Subtract this entry from the totals
			gCSint.bat.playersDmgTots[gCSint.bat.playerGiven[i]][kDmgEndTotIdx] = 
				gCSint.bat.playersDmgTots[gCSint.bat.playerGiven[i]][kDmgEndTotIdx]
				- gCSint.bat.dmgGiven[i]
		--	gCSint.bat.playersDmgTots[v.playerN][kDmgEndTotIdx] = gCSint.bat.playersDmgTots[v.playerN][kDmgEndTotIdx] - v.dmgP
			-- Mark for later removal *We can't do the remove here cause table.remove reindexes the list and bones our loop
			--deleteindex = k
			
			gCSint.bat.startGivenIdx = i + 1
		else
			-- were into the good stuff now, stop
			--oldesttime = v.timestamp
			oldesttime = gCSint.bat.timeStampGiven[i]
			break
		end
	end
	
	--[[
	-- maybe re-impliment this
	
	-- Remove entries from deleteindex down so we don't miss em
	while (deleteindex > 0) do
		table.remove(battledmgGivenTBL, deleteindexTaken)
		deleteindex = deleteindex -1
	end
	]]
		
	
	--
	--	Defensive
	--
	for i = gCSint.bat.startTakenIdx, gCSint.bat.takenCount do
	--for w,y in pairs(battledmgTakenTBL) do
		-- Get rid of old dps entries and adjust the totals
		if ( (curtime - CombatStats_DPSLen) > gCSint.bat.timeStampTaken[i] ) then
			-- Subtract this entry from the totals
			gCSint.bat.playersDmgTots[gCSint.bat.playerTaken[i]][kDmgTakenEndTotIdx] = 
				gCSint.bat.playersDmgTots[gCSint.bat.playerTaken[i]][kDmgTakenEndTotIdx]
				- gCSint.bat.dmgTaken[i]
			-- Mark for later removal *We can't do the remove here cause table.remove reindexes the list and bones our loop
			--deleteindexTaken = w
			gCSint.bat.startTakenIdx = i + 1
		else
			-- were into the good stuff now, stop
			--oldesttimeTaken = y.timestamp
			oldesttimeTaken = gCSint.bat.timeStampTaken[i]
			break
		end
	end
	
	--[[
	-- maybe re impliment this
	
	-- Remove entries from deleteindex down so we don't miss em
	while (deleteindexTaken > 0) do
		table.remove(battledmgTakenTBL, deleteindexTaken)
		deleteindexTaken = deleteindexTaken -1
	end
	]]

		-- NOTE: Everyone calcs off the same oldest time, not their own oldest time.
		--		 This should give more accurate dps on a per group fight basis.
	if ( gCSint.bat.givenCount >= gCSint.bat.startGivenIdx ) then
		--local text = "CurTime - Oldesettime : %.1f"
		--text = format(text,(curtime - oldesttime))
		--cs_print(text)
		
		-- Update player DPS
		if ( gCSint.bat.playersDmgTots[gCSint.bat.yourIdx][kDmgEndTotIdx] ~= nil ) then
			if(gCSint.bat.playersDmgTots[gCSint.bat.yourIdx][kDmgTakenEndTotIdx] ~= nil) then
				gCSint.bat.dpsTakenCalc = (gCSint.bat.playersDmgTots[gCSint.bat.yourIdx][kDmgTakenEndTotIdx] / (curtime - oldesttimeTaken) )
				gCSint.bat.dpsDoneCalc = (gCSint.bat.playersDmgTots[gCSint.bat.yourIdx][kDmgEndTotIdx] / (curtime - oldesttime) )
			else
				gCSint.bat.dpsTakenCalc = 0.0
				gCSint.bat.dpsDoneCalc = (gCSint.bat.playersDmgTots[gCSint.bat.yourIdx][kDmgEndTotIdx] / (curtime - oldesttime) )
			end
			CombatStats_UpdateDynCombatTexts()
		
		end
		
	end

	--
	--	If the data frame is visible
	--	update the selected attack "Real time"
	--
	if (CombatStatsDataFrame:IsVisible()) then
		if( CombatStatsAttackDropDown_GetText() ~= CS_DROPDOWN_SELECT_TEXT) then
			CombatStats_UpdateDetails( CombatStatsAttackDropDown_GetText())
		end
	end
end

function CombatStats_ChkGarbage(typeToChk)
	--cs_print("CombatStats_ChkGarbage")
	typeToChk = typeToChk or 0
	
	if (CombatStats_Config.CombatStats_GC == 0) then
		return
	elseif (CombatStats_Config.CombatStats_GC == kGCafterEOFType) then
		if (typeToChk == kGCafterEOFType) then
			collectgarbage('collect')
			--cs_print("CombatStats_ChkGarbage (A) ---- collect")
		end
		return
	elseif (CombatStats_Config.CombatStats_GC == kGCatRestType) then
		if ((typeToChk == kGCatRestType) and (IsResting())) then
			collectgarbage('collect')
			--cs_print("CombatStats_ChkGarbage (B)---- collect")
		end
		return
	end
	
	-- someday maybe use PLAYER_FLAGS_CHANGED
end

function CombatStats_RegisterAllEvents()
	-- CombatStats_RegisterWhenActiveEvents is called after the setting are loaded if needed
	this:RegisterEvent("VARIABLES_LOADED")
	--this:RegisterEvent("PLAYER_FOCUS_CHANGED")
end

function CombatStats_UpdateWhenActiveRegistery()
	if ( CombatStats_Config.CombatStats_OnOff == 1 ) then
		CombatStats_RegisterWhenActiveEvents()
		CombatStats_UpdateWhenRestingRegistery()
	else
		CombatStats_UnregisterWhenActiveEvents()
	end
end

function CombatStats_UpdateWhenRestingRegistery()
	if (CombatStats_Config.CombatStats_GC == kGCatRestType) then
		this:RegisterEvent("PLAYER_UPDATE_RESTING")
	else
		this:UnregisterEvent("PLAYER_UPDATE_RESTING")
	end
end
			
function CombatStats_RegisterWhenActiveEvents()
	--cs_print("CombatStats_RegisterWhenActiveEvents")
	if (gCSint.areActiveRegister == 1) then return end
	
	this:RegisterEvent("PLAYER_FOCUS_CHANGED")
	this:RegisterEvent("PLAYER_TARGET_CHANGED")

	this:RegisterEvent("CHAT_MSG_COMBAT_SELF_HITS")
	this:RegisterEvent("CHAT_MSG_COMBAT_SELF_MISSES")
	this:RegisterEvent("CHAT_MSG_COMBAT_PET_HITS")
	this:RegisterEvent("CHAT_MSG_COMBAT_PET_MISSES")
	this:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE")
	this:RegisterEvent("CHAT_MSG_SPELL_PET_DAMAGE")
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE")
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE")
	this:RegisterEvent("CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS")
	this:RegisterEvent("CHAT_MSG_COMBAT_HOSTILEPLAYER_MISSES")
	this:RegisterEvent("CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS")
	this:RegisterEvent("CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES")
	this:RegisterEvent("CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE")
	this:RegisterEvent("CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE")
	this:RegisterEvent("CHAT_MSG_SPELL_CREATURE_VS_SELF_BUFF")
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_BUFFS")
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE")
	this:RegisterEvent("CHAT_MSG_SYSTEM")
	this:RegisterEvent("PLAYER_REGEN_ENABLED")
	this:RegisterEvent("PLAYER_REGEN_DISABLED")
	this:RegisterEvent("PLAYER_ENTER_COMBAT")
	this:RegisterEvent("PLAYER_LEAVE_COMBAT")
	this:RegisterEvent("CHAT_MSG_COMBAT_XP_GAIN")
	this:RegisterEvent("CHAT_MSG_COMBAT_LOG_ENEMY")
	this:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH")
	-- Heals
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS")
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS")
	this:RegisterEvent("CHAT_MSG_SPELL_PET_BUFF")
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS")
	this:RegisterEvent("CHAT_MSG_SPELL_SELF_BUFF")
	this:RegisterEvent("CHAT_MSG_SPELL_PARTY_BUFF")
	this:RegisterEvent("CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF")
	this:RegisterEvent("CHAT_MSG_SPELL_ITEM_ENCHANTMENTS")
	
	gCSint.areActiveRegister = 1
end

function CombatStats_UnregisterWhenActiveEvents()
	--cs_print("CombatStats_UnregisterWhenActiveEvents")
	if (gCSint.areActiveRegister == 0) then return end
	
	this:UnregisterEvent("PLAYER_FOCUS_CHANGED")
	this:UnregisterEvent("PLAYER_TARGET_CHANGED")

	this:UnregisterEvent("CHAT_MSG_COMBAT_SELF_HITS")
	this:UnregisterEvent("CHAT_MSG_COMBAT_SELF_MISSES")
	this:UnregisterEvent("CHAT_MSG_COMBAT_PET_HITS")
	this:UnregisterEvent("CHAT_MSG_COMBAT_PET_MISSES")
	this:UnregisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE")
	this:UnregisterEvent("CHAT_MSG_SPELL_PET_DAMAGE")
	this:UnregisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE")
	this:UnregisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE")
	this:UnregisterEvent("CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS")
	this:UnregisterEvent("CHAT_MSG_COMBAT_HOSTILEPLAYER_MISSES")
	this:UnregisterEvent("CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS")
	this:UnregisterEvent("CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES")
	this:UnregisterEvent("CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE")
	this:UnregisterEvent("CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE")
	this:UnregisterEvent("CHAT_MSG_SPELL_CREATURE_VS_SELF_BUFF")
	this:UnregisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_BUFFS")
	this:UnregisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE")
	this:UnregisterEvent("CHAT_MSG_SYSTEM")
	this:UnregisterEvent("PLAYER_REGEN_ENABLED")
	this:UnregisterEvent("PLAYER_REGEN_DISABLED")
	this:UnregisterEvent("PLAYER_ENTER_COMBAT")
	this:UnregisterEvent("PLAYER_LEAVE_COMBAT")
	this:UnregisterEvent("CHAT_MSG_COMBAT_XP_GAIN")
	this:UnregisterEvent("CHAT_MSG_COMBAT_LOG_ENEMY")
	this:UnregisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH")
	-- Heals
	this:UnregisterEvent("CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS")
	this:UnregisterEvent("CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS")
	this:UnregisterEvent("CHAT_MSG_SPELL_PET_BUFF")
	this:UnregisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS")
	this:UnregisterEvent("CHAT_MSG_SPELL_SELF_BUFF")
	this:UnregisterEvent("CHAT_MSG_SPELL_PARTY_BUFF")
	this:UnregisterEvent("CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF")
	this:UnregisterEvent("CHAT_MSG_SPELL_ITEM_ENCHANTMENTS")
	
	this:UnregisterEvent("PLAYER_UPDATE_RESTING")
	
	gCSint.areActiveRegister = 0
end


-- Called from XML

function CombatStats_OnLoad()
	
	-- Special OnLoad for "FuBar"
	if (gCSint.haveFuBar) then
		if (CombatStats.FuBar.Load()) then
			cs_print("FuBar was found")
		end
	end
	
	-- Special OnLoad for "Khaos"
	if ( Khaos ) then
		CombatStats_RegisterKhaos()
	else
	-- Standalone (chatwatch)
		local addonTitle = GetAddOnMetadata( kAddonName, "Title" )
		local addonVersion = GetAddOnMetadata( kAddonName, "Version" )
		
		--DropDownList1:SetClampedToScreen(1);		-- is it needed
		--DropDownList2:SetClampedToScreen(1)
--		CombatStats_Old_ToggleDropDownMenu = ToggleDropDownMenu
--		ToggleDropDownMenu = CombatStats_ToggleDropDownMenu

		SLASH_CSSLASH1 = "/combatstats"
		SLASH_CSSLASH2 = "/cs"
		SlashCmdList["CSSLASH"] = CombatStats_ChatCommandHandler
		
		if ( addonTitle and addonVersion ) then
			DEFAULT_CHAT_FRAME:AddMessage( "|cffffff00"..addonTitle..", v"..addonVersion.."|r loaded. Type |cffffff00"..SLASH_CSSLASH1.."|r for help." )
		else
			DEFAULT_CHAT_FRAME:AddMessage( "|cffffff00"..kAddonName.."|r loaded. Type |cffffff00"..SLASH_CSSLASH1.."|r for help." )
		end
	end

	CombatStats_RegisterAllEvents()
	tinsert(UISpecialFrames, "CombatStatsDataFrame")
	CombatStatsText:SetText(CS_MINIFRAME_DEFAULT_TEXT)
	CombatStats_UpdateVisibility()
end


function CombatStats_ArrTBLItemCount(theTBL)
	-- unknown why table.getn only works on aTable[2] and not aTable["fred"]
	local i
	local aCount=0
	for _,_ in pairs(theTBL) do aCount=aCount+1 end
	return aCount
end

function CombatStats_DumpInternalStats()
	--local num = GetNumAddOns()
	local mem = -1
	local cpu = -1
	local count
	local info1, info2
	local text
	--local _, _, _, khaosEnabled, _, _, _ = GetAddOnInfo("Khaos")
	--local _, _, _, titanEnabled, _, _, _ = GetAddOnInfo("Titan")
	--local _, _, _, infobarEnabled, _, _, _ = GetAddOnInfo("InfoBar")
	
	cs_print("CombatStats")
	
	-- get memory & CPU usage
	UpdateAddOnMemoryUsage()
	mem = GetAddOnMemoryUsage(kAddonName)
	cpu = GetAddOnCPUUsage(kAddonName)
	--[[
	for i = 1, num do
		if (kAddonName == GetAddOnInfo(i)) then 
			mem = GetAddOnMemoryUsage(i)
			cpu = GetAddOnCPUUsage(i)
			break
		end
	end	]]
	cs_print(" Memory used: "..format("%.1f", mem).."(KB), CPU usage: "..cpu)
	
	cs_print(" Outside addon integration")
	--if (khaosEnabled) then cs_print("   Khaos integration is active") 
	if (gCSint.haveKhaos) then cs_print("   Khaos integration is active")
	else cs_print("   Khaos is not installed or not active") end
	--if (titanEnabled) then cs_print("   Titan integration is active")
	if (gCSint.haveTitan) then cs_print("   Titan integration is active")
	else cs_print("   Titan is not installed or not active") end
	--if (infobarEnabled) then cs_print("   InfoBar integration is active")
	if (gCSint.haveInfoBar) then cs_print("   InfoBar integration is active")
	else cs_print("   InfoBar is not installed or not active") end
	info1, info2 = CombatStats.FuBar.CheckDependences()
	if (info1) then
		text = "   FuBar is active BUT: "
		if (info1["FubarPlugMissing"]) then text = text.."FuBarPlugin library, " end
		if (info1["FubarAceMissing"]) then text = text.."AceAddon library, " end
		if (info1["FubarTabletMissing"]) then text = text.."AceTablet library, " end
		text = text.."are missing"
	else
		cs_print("   FuBar is not installed or not active")
	end

	-- storage counts
	cs_print(" CombatStats_Config item count: "..CombatStats_ArrTBLItemCount(CombatStats_Config))
	cs_print(" gCSint.bat.playersDmgTots item count: "..CombatStats_ArrTBLItemCount(gCSint.bat.playersDmgTots))
	cs_print(" gCS item count: "..CombatStats_ArrTBLItemCount(gCS))
	cs_print(" gCSint item count: "..CombatStats_ArrTBLItemCount(gCSint))
--	cs_print(" attackNames idx count: "..CombatStats_ArrTBLItemCount(attackNames))
--	cs_print(" groupNames idx count: "..CombatStats_ArrTBLItemCount(groupNames))
	cs_print(" specialAttacks item count: "..CombatStats_ArrTBLItemCount(specialAttacks))
--	cs_print(" specialAttackLog item count: "..CombatStats_ArrTBLItemCount(specialAttackLog))

end

function CombatStats_DumpConfigVariables()
	local aValue
	
	cs_print("CombatStats_Config data:")

	aValue = CombatStats_Config.CombatStats_OnOff
	if(aValue == nil) then aValue = "nil" end
	cs_print(" CombatStats_OnOff: "..aValue)
	aValue = CombatStats_Config.CombatStats_HideOnNoTarget
	if(aValue == nil) then aValue = "nil" end
	cs_print(" CombatStats_HideOnNoTarget: "..aValue)
	aValue = CombatStats_Config.CombatStats_EndOfFight
	if(aValue == nil) then aValue = "nil" end
	cs_print(" CombatStats_EndOfFight: "..aValue)
	aValue = CombatStats_Config.CombatStats_UseMouseOver
	if(aValue == nil) then aValue = "nil" end
	cs_print(" CombatStats_UseMouseOver: "..aValue)
	aValue = CombatStats_Config.CombatStats_Hide
	if(aValue == nil) then aValue = "nil" end
	cs_print(" CombatStats_Hide: "..aValue)
	aValue = CombatStats_Config.CombatStats_HideTtip
	if(aValue == nil) then aValue = "nil" end
	cs_print(" CombatStats_HideTtip: "..aValue)
	aValue = CombatStats_Config.CombatStats_AttackMenuSort
	if(aValue == nil) then aValue = "nil" end
	cs_print(" CombatStats_AttackMenuSort: "..aValue)
	aValue = CombatStats_Config.CombatStats_AttackMenu1stGrping
	if(aValue == nil) then aValue = "nil" end
	cs_print(" CombatStats_AttackMenu1stGrping: "..aValue)
	aValue = CombatStats_Config.CombatStats_GC
	if(aValue == nil) then aValue = "nil" end
	cs_print(" CombatStats_GC: "..aValue)
end


function CombatStats_DumpArrayString(theArray, thePreLabel)
	local aString = ""
	thePreLabel = thePreLabel or "idx"
	
	--for i=1, table.getn(theArray) do
	for i,j in pairs(theArray) do
		aString = aString..thePreLabel.."["..i.."] = "..theArray[i]..", "
	end
	
	return aString
end


function CombatStats_CheckLoadedVariables()
	local 	aCounter
	local	removeCount = 0
	
	CombatStats.ComParse.Load()
	
	--[[ add missing config record fields ]]
	if(CombatStats_Config.CombatStats_OnOff == nil) then
		CombatStats_Config.CombatStats_OnOff = 1
	end
	-- toggle twice back to the correct state (done to handle the event register)
	CombatStats_SetEnable()
	CombatStats_SetEnable()
		
	if(CombatStats_Config.CombatStats_HideOnNoTarget == nil) then
		CombatStats_Config.CombatStats_HideOnNoTarget = 0
	end
	if(CombatStats_Config.CombatStats_EndOfFight == nil) then
		CombatStats_Config.CombatStats_EndOfFight = 0
	end
	if(CombatStats_Config.CombatStats_UseMouseOver == nil) then
		CombatStats_Config.CombatStats_UseMouseOver = 0
	end
	if(CombatStats_Config.CombatStats_Hide == nil) then
		CombatStats_Config.CombatStats_Hide = 0
	end
	if(CombatStats_Config.CombatStats_HideTtip == nil) then
		CombatStats_Config.CombatStats_HideTtip = 0
	end
	if(CombatStats_Config.CombatStats_AttackMenuSort == nil) then
		CombatStats_Config.CombatStats_AttackMenuSort = 0
	end
	if(CombatStats_Config.CombatStats_AttackMenu1stGrping == nil) then
		CombatStats_Config.CombatStats_AttackMenu1stGrping = 1
	end
	--		CombatStats.AttackHistDB.setGrpSortType(CombatStats_Config.CombatStats_AttackMenu1stGrping);		-- FIX ME!!!
	if(CombatStats_Config.CombatStats_GC == nil) then
		CombatStats_Config.CombatStats_GC = 0
	end
	if(CombatStats_Config.CombatStats_MaxUpPS == nil) then
		CombatStats_Config.CombatStats_MaxUpPS = 30
	end
	
	if(gCS.currentAttack == nil) then
		gCS.currentAttack = CS_TEXT_NA
	end
	
	-- ONLY NEEDED PRE VERSION ?? but definitely ONLY needed for version 4.49 on
	if (gCS.dataVersion == nil or gCS.dataVersion < 449) then
		CombatStats.Import.DataPre449()
	end
	
	gCS.dataVersion = kAddonVerNum
end

function CombatStats_ResetPos()
	local frame;
	
	frame = CombatStatsFrame;
	-- reset the small DPS frame
	if (frame ~= nil) then
		frame:StopMovingOrSizing();
		frame.isMoving = false;
		frame:ClearAllPoints();
		frame:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", 240, -25);
	
		-- Make sure the window is visible, also.
		CombatStats_SetTarget(0);
		CombatStats_SetHide(0);
		frame:Show();
	--else
		--cs_print("CombatStats_ResetPos : Error getting small DPS frame.");
	end
	
	frame = CombatStatsDataFrame;
	-- reset the detail stats frame
	if (frame ~= nil) then
		frame:StopMovingOrSizing();
		frame.isMoving = false;
		frame:ClearAllPoints();
		frame:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", 30, -150);
	
		-- Make sure the window is visible, too.
		CombatStatsText_ShowFrame();
	--else
		--cs_print("CombatStats_ResetPos : Error getting detail stats frame.");
	end
	
end


--
--	Add the damage to the DPS table
--

function CombatStats_AddDPSEntry(timeStamp, p, d)
	local playerIdx = 0
	
	-- get player index
	for i=1, table.getn(gCSint.bat.playersTBL), 1 do
		if(gCSint.bat.playersTBL[i] == p) then
			playerIdx = i
			break
		end
	end
	-- add player to table if it does not exist
	if (playerIdx == 0) then
		-- add player index
		table.insert(gCSint.bat.playersTBL, p)
		playerIdx = table.getn(gCSint.bat.playersTBL)
	end
	
	if (playerIdx == 0) then cs_print("ERR: playerIdx not set") return end
	
	gCSint.bat.givenCount = gCSint.bat.givenCount + 1
	gCSint.bat.timeStampGiven[gCSint.bat.givenCount] = timeStamp
	gCSint.bat.playerGiven[gCSint.bat.givenCount] = playerIdx
	gCSint.bat.dmgGiven[gCSint.bat.givenCount] = d

	-- add data totals if this is a new player entry
	if (gCSint.bat.playersDmgTots[playerIdx] == nil) then
		cs_print("new player index: "..playerIdx..", for "..p)
		gCSint.bat.playersDmgTots[playerIdx] = {0,0,0,0}
		--gCSint.bat.playersDmgTots[playerIdx] = { dmgEndTot = 0, dmgTakenEndTot = 0,
		--						dmgTot = 0, dmgTakenTot = 0 }
	end
	
	-- Keep running totals on group members to shorten calculation times
	-- We'll subtract off the expired entries during OnUpdate
	if (gCSint.bat.playersDmgTots[playerIdx][kDmgEndTotIdx] == nil) then
		--cs_print("gCSint.bat.playersDmgTots[playerIdx][kDmgEndTotIdx] == nil")
		gCSint.bat.playersDmgTots[playerIdx][kDmgEndTotIdx] = d
		gCSint.bat.playersDmgTots[playerIdx][kDmgTotIdx] = d
	else
		--cs_print("gCSint.bat.playersDmgTots[playerIdx][kDmgEndTotIdx] == " ..gCSint.bat.playersDmgTots[playerIdx][kDmgEndTotIdx] )
		gCSint.bat.playersDmgTots[playerIdx][kDmgEndTotIdx] = gCSint.bat.playersDmgTots[playerIdx][kDmgEndTotIdx] + d
		gCSint.bat.playersDmgTots[playerIdx][kDmgTotIdx] = gCSint.bat.playersDmgTots[playerIdx][kDmgTotIdx] + d
		--cs_print("AFTER : gCSint.bat.playersDmgTots[playerIdx][kDmgEndTotIdx] == " ..gCSint.bat.playersDmgTots[playerIdx][kDmgEndTotIdx] )
	end
	--cs_print("Total : " ..gCSint.bat.playersDmgTots[playerIdx][kDmgEndTotIdx])
end


--
--	Add damage taken to the DPS table
--

function CombatStats_AddDefDPSEntry(timeStamp, p, d)
	local playerIdx = 0
	
	--cs_print("Adding Entry[" .. p .. "] = " .. d);	-- debug
	
	-- get player index
	for i=1, table.getn(gCSint.bat.playersTBL), 1 do
		if(gCSint.bat.playersTBL[i] == p) then
			playerIdx = i
			break
		end
	end
	-- add player to table if it does not exist
	if (playerIdx == 0) then
		-- add player index
		table.insert(gCSint.bat.playersTBL, p)
		playerIdx = table.getn(gCSint.bat.playersTBL)
	end
	
	if (playerIdx == 0) then cs_print("ERR: playerIdx not set") end
	
	gCSint.bat.takenCount = gCSint.bat.takenCount + 1
	gCSint.bat.timeStampTaken[gCSint.bat.takenCount] = timeStamp
	gCSint.bat.playerTaken[gCSint.bat.takenCount] = playerIdx
	gCSint.bat.dmgTaken[gCSint.bat.takenCount] = d
	
	-- add data totals if this is a new player entry
	if (gCSint.bat.playersDmgTots[playerIdx] == nil) then
		gCSint.bat.playersDmgTots[playerIdx] = {0,0,0,0}
		--gCSint.bat.playersDmgTots[playerIdx] = { dmgEndTot = 0, dmgTakenEndTot = 0,
		--						dmgTot = 0, dmgTakenTot = 0 }
	end
	
	-- Keep running totals on group members to shorten calculation times
	-- We'll subtract off the expired entries during OnUpdate
	if (gCSint.bat.playersDmgTots[playerIdx][kDmgTakenEndTotIdx] == nil) then
		--cs_print("gCSint.bat.playersDmgTots[playerIdx][kDmgEndTotIdx] == nil")
		gCSint.bat.playersDmgTots[playerIdx][kDmgTakenEndTotIdx] = d
		gCSint.bat.playersDmgTots[playerIdx][kDmgTakenTotIdx] = d
	else
		--cs_print("gCSint.bat.playersDmgTots[playerIdx][kDmgEndTotIdx] == " ..gCSint.bat.playersDmgTots[playerIdx][kDmgEndTotIdx] )
		gCSint.bat.playersDmgTots[playerIdx][kDmgTakenEndTotIdx] = gCSint.bat.playersDmgTots[playerIdx][kDmgTakenEndTotIdx] + d
		gCSint.bat.playersDmgTots[playerIdx][kDmgTakenTotIdx] = gCSint.bat.playersDmgTots[playerIdx][kDmgTakenTotIdx] + d
		--cs_print("AFTER : gCSint.bat.playersDmgTots[playerIdx][kDmgEndTotIdx] == " ..gCSint.bat.playersDmgTots[playerIdx][kDmgEndTotIdx] )
	end
	--cs_print("Total : " ..gCSint.bat.playersDmgTots[playerIdx][kDmgEndTotIdx])
end


--	Show the stats collected

function CombatStatsText_OnClick()
	if (CombatStatsDataFrame:IsVisible()) then
		CombatStatsDataFrame:Hide()
	else
		if (CombatStats_Config.CombatStats_UseMouseOver == 0) then
			CombatStatsText_ShowFrame()
		end
	end
end

function CombatStatsText_OnEnter()
	if (CombatStats_Config.CombatStats_UseMouseOver == 1) then
		CombatStatsText_ShowFrame()
	end
end

--
-- set accessors
--
function CombatStats_SetEnable(theValue)
	-- nil theValue is for toggling
	--cs_print("CombatStats_SetEnable")
	
	if (theValue ~= nil) then
		if (CombatStats_Config.CombatStats_OnOff == theValue) then return end
		CombatStats_Config.CombatStats_OnOff = theValue
	else
		if (CombatStats_Config.CombatStats_OnOff == 0) then
			CombatStats_Config.CombatStats_OnOff = 1
		else
			CombatStats_Config.CombatStats_OnOff = 0
		end
	end

	-- value has been changed
	if (CombatStats_Config.CombatStats_OnOff == 0) then
		CombatStats_SetDynCombatTexts(CS_MINIFRAME_DISABLE_TEXT)
		--CombatStats_SetCombatText(CS_MINIFRAME_DISABLE_TEXT,1)
		--CombatStats_SetDynCombatTexts(CS_MINIFRAME_DISABLE_TEXT)
		--CombatStatsFrame:Hide()
	else
		CombatStats_UpdateDynCombatTexts()
		--CombatStats_SetDynCombatTexts(CS_MINIFRAME_DEFAULT_TEXT)
		--CombatStats_SetCombatText(nil,2)
	end
--	CombatStats_UpdateDynCombatTexts()
	CombatStats_UpdateWhenActiveRegistery()
	CombatStats_UpdateVisibility()
end

function CombatStats_SetTarget(theValue)
	-- nil theValue is for toggling
	
	if (theValue ~= nil) then
		CombatStats_Config.CombatStats_HideOnNoTarget = theValue
	else
		if (CombatStats_Config.CombatStats_HideOnNoTarget == 0) 
			then CombatStats_Config.CombatStats_HideOnNoTarget = 1
			else CombatStats_Config.CombatStats_HideOnNoTarget = 0
		end
	end
	
	CombatStats_UpdateVisibility()
end

function CombatStats_SetMouseOver(theValue)
	-- nil theValue is for toggling
	
	if (theValue ~= nil) then
		CombatStats_Config.CombatStats_UseMouseOver = theValue
	else
		if (CombatStats_Config.CombatStats_UseMouseOver == 0) 
			then CombatStats_Config.CombatStats_UseMouseOver = 1
			else CombatStats_Config.CombatStats_UseMouseOver = 0
		end
	end
	
	CombatStats_UpdateVisibility()
end

function CombatStats_SetEndFight(theValue)
	-- nil theValue is for toggling
	
	if (theValue ~= nil) then
		CombatStats_Config.CombatStats_EndOfFight = theValue
	else
		if (CombatStats_Config.CombatStats_EndOfFight == 0) 
			then CombatStats_Config.CombatStats_EndOfFight = 1
			else CombatStats_Config.CombatStats_EndOfFight = 0
		end
	end
	
	CombatStats_UpdateVisibility()
end

function CombatStats_SetHide(theValue)
	-- nil theValue is for toggling
	
	if (theValue ~= nil) then
		CombatStats_Config.CombatStats_Hide = theValue
	else
		if (CombatStats_Config.CombatStats_Hide == 0) 
			then CombatStats_Config.CombatStats_Hide = 1
			else CombatStats_Config.CombatStats_Hide = 0
		end
	end
	
	CombatStats_UpdateVisibility()
end

function CombatStats_SetHideTtip(theValue)
	-- nil theValue is for toggling
	
	if (theValue ~= nil) then
		CombatStats_Config.CombatStats_HideTtip = theValue
	else
		if (CombatStats_Config.CombatStats_HideTtip == 0) 
			then CombatStats_Config.CombatStats_HideTtip = 1
			else CombatStats_Config.CombatStats_HideTtip = 0
		end
	end
	
	CombatStats_UpdateVisibility()
end

function CombatStats_UpdateFramePer(theValue)
	--cs_print("CombatStats_UpdateFramePer")
	theValue = tonumber(theValue) or 10
	
	if ((theValue == 0) or (theValue < .001)) then
		theValue = .001
	elseif (theValue > 99) then
		theValue = 99
	end
	
	CombatStats_Config.CombatStats_MaxUpPS = theValue
end

function CombatStats_SetMaxUPS()
	StaticPopup_Show("CombatStats_GUI_Get_Max_UpdatePreSec");
end
							
--
-- WoW std function overrides
--

function CombatStats_ToggleDropDownMenu(level, value, dropDownFrame, anchorName, xOffset, yOffset)
	local dropDownFrameName
	if (CombatStats_Old_ToggleDropDownMenu) then
		level = level or 1
		-- Only modify the level 1 behavior
		if ( level == 1 ) then
			if ( not dropDownFrame ) then
				dropDownFrame = this:GetParent()
			end
			dropDownFrameName = dropDownFrame:GetName()
			-- If there's no specified anchorName then use left side of the dropdown menu
			if ( not anchorName ) then
				anchorName = dropDownFrameName.."Left"
			end
			if ( not xOffset or not yOffset ) then
				xOffset = 5
				yOffset = 25
			end
		end
		CombatStats_Old_ToggleDropDownMenu(level, value, dropDownFrame, anchorName, xOffset, yOffset)
	end
end

function CombatStats_ClearEntireTBL(theTable)
	for k,v in pairs(theTable) do
		theTable[k] = 0;
	end
end

function CombatStats_NilEntireTBL(theTable)
	for k=table.getn(theTable), 1, -1 do
		theTable[k] = nil;
	end
end

function CombatStats_RemoveEntiresTBL(theTable)
	for i=table.getn(theTable), 1, -1 do
		table.remove(theTable ,i);
	end
end



function cs_print(msg, r, g, b, frame, id)
	local info = ChatTypeInfo["SYSTEM"]
	r = r or 1.0		-- info.r
	g = g or 1.0		-- info.g
	b = b or 1.0		-- info.b
	id = id or info.id
	frame = frame or DEFAULT_CHAT_FRAME
	
	frame:AddMessage(msg,r,g,b,id)
	
--[[	
	if (not frame) then frame = DEFAULT_CHAT_FRAME; end
	
	if (frame) then
		if (not id) then
			frame:AddMessage(msg,r,g,b)
		else
			frame:AddMessage(msg,r,g,b,id)
		end
	end
	]]
end




--[[
						The Titan, InfoBar, Sea, Cosmos and Khaos
						changes and additions are all from
				http://www.petewood.info/Interface/AddOns/CombatStats/
	The revision history for them as follows
	Rev 4.2 - Removed legacy Cosmos config
	Rev 3.971 - Removed the "DPS : " from the titan panel
	Rev 3.97	- 2 fixes,	1) The sCombatStats.... error
							2) The error in the de.lua file
	Rev 3.96	- German localization translations added
	Rev 3.94	- Fixed the error if you weren't using Titan
	Rev 3.94	- Fixed the not updating on TitanPane while hidden bug
	Rev 3.92	- Added TitanPanel plug-in support
			- Added a slash command to hide the tooltip
	Rev 3.91	- HUGE bugfix. The infobar stuff was causing the OnEvent to get ran twice
			 causing your damage to be recorded twice, as such the DPS value was doubled
	Rev 3.9		- Removed mouseover stats pop up when used w/infobar
			- Added a /cs showstats command to show the attack stats
	Rev 3.8		- Removed any Sea calls, this will get rid of any of the UIDROPDOWNxxxx errors
			 people may have been having [I hadn't noticed because I don't use Sea]
			- Implemented the right click to drag the window that had been added by the CTMod folks
			- Re-did the slash command handler so it actually works, the slash commands are now
				enable - 'Enables CombatStats'
				disable - 'Disables CombatStats'
				showonlywhentarget - 'Only show DPS meter when you have a target'
				showalways - 'Only show DPS meter always'
				statsonmouseover - 'Show detailed window on mouseover'
				statsonclick - 'Show detailed window on mouseclick'
				endoffight - 'Show end of fight information'
				noendoffight - 'Don't show end of fight information'
				hide - 'Hide the frame '
				show - 'Show the frame '
				reset - 'Reset all stats to 0'
]]


--
--	InfoBar Items
--

function CombatStatsIB_OnLoad()
	this.info = 
	{
		name=kAddonName,
		version=kAddonVerNum,
		update=CombatStats_OnUpdate,
		tooltip=CombatStatsIB_Tooltip,
	}
	
	CombatStats_RegisterAllEvents()
	
	if(CombatStatsIBFrame:IsVisible()) then
		CombatStatsIBFrameCenteredText:SetText(CS_MINIFRAME_DEFAULT_TEXT)
	end
end

function CombatStatsIB_OnClick()
	CombatStatsText_OnClick()
end

function CombatStatsIB_OnEnter()
	--InfoBar_ShowTooltip();
	--GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
	--GameTooltip:SetText("Right-click to show stats.");
	InfoBar_ShowPluginTooltip(this);
	
	CombatStatsText_OnEnter()
end

function CombatStatsIB_Tooltip()
	return CS_TT_ADDON_TEXT_INFOBAR
end

--
--	Titan Items
--

function CombatStatsTB_OnLoad()
	--if (gCSint.haveTitan == false) then return end	-- not needed because of the 'TitanPanelBarButton~=nil' check in xml
	
	--CombatStats_RegisterAllEvents()
	
	this.registry = {
		id = CombatStatsTB_ID,
		category = "Combat",
		version = gCSint.addonVersion,
		menuText = gCSint.addonTitle,
		buttonTextFunction = "CombatStatsTB_GetButtonText",
		tooltipTitle = gCSint.addonTitle, 
		tooltipTextFunction = "CombatStatsTB_GetTooltipText",
		frequency = CombatStatsTB_DPS_FREQUENCY,
		updateType = TITAN_PANEL_UPDATE_BUTTON,
		savedVariables = {
			OnOff = 0,				-- kCSsetEnable
			HideOnNoTarget = 0,		-- kCSsetTarget
			EndOfFight = 0,			-- kCSsetEndofFight
			UseMouseOver = 0,		-- kCSsetMouseOver
			Hide = 0,				-- kCSsetHide
			HideTtip = 0,			-- kCSsetHideTTip
			ShowIcon = 1,
			ShowLabelText = TITAN_NIL,
			ShowColoredText = 1,
		},
	}
	
	Org_TitanPanelRightClickMenu_ToggleVar = TitanPanelRightClickMenu_ToggleVar
	TitanPanelRightClickMenu_ToggleVar = New_TitanPanelRightClickMenu_ToggleVar
end

-- Titan function override
function New_TitanPanelRightClickMenu_ToggleVar()
	Org_TitanPanelRightClickMenu_ToggleVar()
	-- sync data setting
	CombatStatsTB_Update_CS_Settings_From_Titan()
	--CombatStats_DumpConfigVariables()		-- DEBUG
end

function CombatStatsTB_GetButtonText(id)
	local button,id = TitanUtils_GetButton(id,true)

	if (CombatStats_Config.CombatStats_OnOff == 0) then return CS_MINIFRAME_DISABLE_TEXT end
	
	if (gCSint.bat.dpsDoneCalc ~= 0 or gCSint.bat.dpsTakenCalc ~= 0) then
		if (gCSint.bat.dpsDoneCalc > 10) then
			return format("|cffff2020\%.1f|cffffd200\ / |cff20ff20\%.1f", gCSint.bat.dpsTakenCalc, gCSint.bat.dpsDoneCalc)
		else
			return format("|cffff2020\%.2f|cffffd200\ / |cff20ff20\%.2f", gCSint.bat.dpsTakenCalc, gCSint.bat.dpsDoneCalc)
		end
	end
	
	return CS_MINIFRAME_DEFAULT_TEXT
end

function CombatStatsTB_GetTooltipText()
	return CS_TB_TOOLTIP_TEXT
end

function TitanPanelRightClickMenu_PrepareCombatStatsMenu()
	-- sync data setting
	CombatStatsTB_Update_Titan_From_CS_Settings()
	
	TitanPanelRightClickMenu_AddTitle(gCSint.addonTitle)
	TitanPanelRightClickMenu_AddToggleVar(CS_CMD_ENABLE_OLD_TEXT, CombatStatsTB_ID, kCSsetEnable)
	TitanPanelRightClickMenu_AddToggleVar(CS_CMD_TARGET_OLD_TEXT, CombatStatsTB_ID, kCSsetTarget)
	TitanPanelRightClickMenu_AddToggleVar(CS_CMD_MOUSEOVER_OLD_TEXT, CombatStatsTB_ID, kCSsetMouseOver)
	TitanPanelRightClickMenu_AddToggleVar(CS_CMD_ENDFLIGHT_OLD_TEXT, CombatStatsTB_ID, kCSsetEndofFight)
	TitanPanelRightClickMenu_AddToggleVar(CS_CMD_HIDE_OLD_TEXT, CombatStatsTB_ID, kCSsetHide)
	TitanPanelRightClickMenu_AddToggleVar(CS_CMD_TOOLTIP_OLD_TEXT, CombatStatsTB_ID, kCSsetHideTTip)
	TitanPanelRightClickMenu_AddSpacer()
	TitanPanelRightClickMenu_AddCommand(CS_CMD_SHOWSTAT_TEXT, CombatStatsTB_ID, "CombatStatsText_ShowFrame")
	TitanPanelRightClickMenu_AddSpacer()
	TitanPanelRightClickMenu_AddCommand(CS_CMD_RESET_TEXT, CombatStatsTB_ID, "CombatStats_Reset")
	TitanPanelRightClickMenu_AddCommand(CS_CMD_RESETWIN_TEXT, CombatStatsTB_ID, "CombatStats_ResetPos")
	TitanPanelRightClickMenu_AddCommand(CS_CMD_SETMAXUPS_TEXT, CombatStatsTB_ID, "CombatStats_SetMaxUPS")
	
	--TitanPanelRightClickMenu_AddSpacer();
	--TitanPanelRightClickMenu_AddToggleIcon(CombatStatsTB_ID);
	--TitanPanelRightClickMenu_AddToggleLabelText(CombatStatsTB_ID);
	--TitanPanelRightClickMenu_AddToggleColoredText(CombatStatsTB_ID);
	
	TitanPanelRightClickMenu_AddSpacer()
	TitanPanelRightClickMenu_AddCommand(TITAN_PANEL_MENU_HIDE, CombatStatsTB_ID, TITAN_PANEL_MENU_FUNC_HIDE)
end

function CombatStatsTB_OnClick(arg1)
	if (arg1 == "RightButton") then
		TitanPanelButton_OnClick(arg1);
	else		
		-- keep in sync with CombatStatsText_OnClick()
		if (CombatStatsDataFrame:IsVisible()) then
			CombatStatsDataFrame:Hide()
		else
			CombatStatsText_ShowFrame()
		end
	end
end
							
function CombatStatsTB_Update_CS_Settings_From_Titan()
	local aSetting
	
	aSetting = TitanGetVar(CombatStatsTB_ID, kCSsetEnable)
	if (aSetting == nil or aSetting == TITAN_NIL) then aSetting = 0 end
	CombatStats_SetEnable(aSetting)
	
	aSetting = TitanGetVar(CombatStatsTB_ID, kCSsetTarget)
	if (aSetting == nil or aSetting == TITAN_NIL) then aSetting = 0 end
	CombatStats_SetTarget(aSetting)
	
	aSetting = TitanGetVar(CombatStatsTB_ID, kCSsetEndofFight)
	if (aSetting == nil or aSetting == TITAN_NIL) then aSetting = 0 end
	CombatStats_SetEndFight(aSetting)
	
	aSetting = TitanGetVar(CombatStatsTB_ID, kCSsetMouseOver)
	if (aSetting == nil or aSetting == TITAN_NIL) then aSetting = 0 end
	CombatStats_SetMouseOver(aSetting)
	
	aSetting = TitanGetVar(CombatStatsTB_ID, kCSsetHide)
	if (aSetting == nil or aSetting == TITAN_NIL) then aSetting = 0 end
	CombatStats_SetHide(aSetting)
	
	aSetting = TitanGetVar(CombatStatsTB_ID, kCSsetHideTTip)
	if (aSetting == nil or aSetting == TITAN_NIL) then aSetting = 1 else aSetting = 0 end
	CombatStats_SetHideTtip(aSetting)
	
	--CombatStats_UpdateVisibility()
	--[[ NOTE: TitanSetVar & TitanGetVar toggle between nil and 1 (this is what I finding with v3.0.5) ]]
end

function CombatStatsTB_Update_Titan_From_CS_Settings()
	-- CombatStats_Config.xxx should never be 'nil'
	
	if ((CombatStats_Config.CombatStats_OnOff~= nil) and CombatStats_Config.CombatStats_OnOff == 1) then
		TitanSetVar(CombatStatsTB_ID, kCSsetEnable, 1) else TitanSetVar(CombatStatsTB_ID, kCSsetEnable, TITAN_NIL) end
	if ((CombatStats_Config.CombatStats_HideOnNoTarget~= nil) and CombatStats_Config.CombatStats_HideOnNoTarget == 1) then
		TitanSetVar(CombatStatsTB_ID, kCSsetTarget, 1) else TitanSetVar(CombatStatsTB_ID, kCSsetTarget, TITAN_NIL) end
	if ((CombatStats_Config.CombatStats_EndOfFight~= nil) and CombatStats_Config.CombatStats_EndOfFight == 1) then
		TitanSetVar(CombatStatsTB_ID, kCSsetEndofFight, 1) else TitanSetVar(CombatStatsTB_ID, kCSsetEndofFight, TITAN_NIL) end
	if ((CombatStats_Config.CombatStats_UseMouseOver~= nil) and CombatStats_Config.CombatStats_UseMouseOver == 1) then
		TitanSetVar(CombatStatsTB_ID, kCSsetMouseOver, 1) else TitanSetVar(CombatStatsTB_ID, kCSsetMouseOver, TITAN_NIL) end
	if ((CombatStats_Config.CombatStats_Hide~= nil) and CombatStats_Config.CombatStats_Hide == 1) then
		TitanSetVar(CombatStatsTB_ID, kCSsetHide, 1) else TitanSetVar(CombatStatsTB_ID, kCSsetHide, TITAN_NIL) end
	if ((CombatStats_Config.CombatStats_HideTtip~= nil) and CombatStats_Config.CombatStats_HideTtip == 1) then
		TitanSetVar(CombatStatsTB_ID, kCSsetHideTTip, TITAN_NIL) else TitanSetVar(CombatStatsTB_ID, kCSsetHideTTip, 1) end	-- NOTE the diff on this line
	
	--[[ NOTE: TitanSetVar & TitanGetVar toggle between nil and 1 (this is what I finding with v3.0.5) ]]
end

--
--	Khaos Items
--

function CombatStats_RegisterKhaos()
	--cs_print("CombatStats_RegisterKhaos",0.0, 1.0, 0.0);
	local localKhaosParseTree = {
		-- String keywords branch into other trees
		["keyword"] = {localKhaosParseTree}; -- IMPORTANT NOTE: KEYWORDS MUST BE LOWERCASE
		-- the default keyword is a special case
		default = {localKhaosParseTree}; -- Will get executed only if nothing else matches
		enable = {callback = CombatStats_ChatCommandHandler;};
		disable = {callback = CombatStats_ChatCommandHandler;};
		showonlywhentarget = {callback = CombatStats_ChatCommandHandler;};
		showalways = {callback = CombatStats_ChatCommandHandler;};
		statsonmouseover = {callback = CombatStats_ChatCommandHandler;};
		statsonclick = {callback = CombatStats_ChatCommandHandler;};
		endoffight = {callback = CombatStats_ChatCommandHandler;};
		noendoffight = {callback = CombatStats_ChatCommandHandler;};
		hide = {callback = CombatStats_ChatCommandHandler;};
		show = {callback = CombatStats_ChatCommandHandler;};
		hidettip = {callback = CombatStats_ChatCommandHandler;};
		showstats = {callback = CombatStats_ChatCommandHandler;};
		reset = {callback = CombatStats_ChatCommandHandler;};
	};
	local localKhaosSlashCommand = {
		id = "CombatStatsCommands";
		commands = {"/combatstats", "/cs"};
		parseTree = { localKhaosParseTree };
		helpText = "CombatStats functions.";
	};
	Khaos.registerOptionSet(
		"combat",
		{
			id = kAddonName;
			text = COMBATSTATS_CONFIG_HEADER;
			helptext = COMBATSTATS_CONFIG_HEADER_INFO;
			difficulty = 1;
			default = true;
			-- commands = localKhaosSlashCommand;
			options = {
				{
					id = "Header";
					type = K_HEADER;
					text = COMBATSTATS_CONFIG_HEADER;
					helptext = COMBATSTATS_CONFIG_HEADER_INFO;
					difficulty = 1;
					default = false;
				};
				{
					id = "Enable";
					type = K_TEXT;
					text = COMBATSTATS_CONFIG_ONOFF;
					helptext = COMBATSTATS_CONFIG_ONOFF_INFO;
					difficulty = 1;
					check = true;
					callback = function(state)
						if (state.checked) then
							CombatStats_SetEnable(1);
						else
							CombatStats_SetEnable(0);
						end
					end;
					feedback = function(state)
						if ( state.checked ) then
							return 'CombatStats is enabled.';
						else
							return 'CombatStats is disabled.';
						end
					end;
					default = {checked = true};
					disabled = {checked = false};
				};
				{
					id = "COMBATSTATS_CONFIG_USEMOUSEOVER";
					type = K_TEXT;
					check = true;
					text = COMBATSTATS_CONFIG_USEMOUSEOVER;
					helptext = COMBATSTATS_CONFIG_USEMOUSEOVER_INFO;
					difficulty = 2;
					feedback = function(state)
						if ( state.checked ) then
							return "Display on mouseover.";
						else
							return "Don't display on mouseover.";
						end
					end;
					callback = function(state)
						if (state.checked) then
							CombatStats_SetMouseOver(1);
						else
							CombatStats_SetMouseOver(0);
						end
					end;
					default = {checked = false};
					disabled = {checked = false};
				};
				{
					id = "COMBATSTATS_HIDEONNOTARGET_ONOFF";
					type = K_TEXT;
					check = true;
					text = COMBATSTATS_CONFIG_HIDEONNOTARGET;
					helptext = COMBATSTATS_CONFIG_HIDEONNOTARGET_INFO;
					difficulty = 2;
					feedback = function(state)
						if ( state.checked ) then
							return "Will hide on no target.";
						else
							return "Will not hide on no target.";
						end
					end;
					callback = function(state)
						if (state.checked) then
							CombatStats_SetTarget(1);
						else
							CombatStats_SetTarget(0);
						end
					end;
					default = {checked = false};
					disabled = {checked = false};
				};
				{
					id = "COMBATSTATS_CONFIG_ENDOFFIGHT";
					type = K_TEXT;
					check = true;
					text = COMBATSTATS_CONFIG_ENDOFFIGHT;
					helptext = COMBATSTATS_CONFIG_ENDOFFIGHT_INFO;
					difficulty = 2;
					feedback = function(state)
						if ( state.checked ) then
							return "Will display stats at end of fight.";
						else
							return "Will not display stats at end of fight.";
						end
					end;
					callback = function(state)
						if ( state.checked ) then
							CombatStats_SetEndFight(1);
						else
							CombatStats_SetEndFight(0);
						end
					end;
					default = {checked = false;slider = 0};
					disabled = {checked = false;slider = 0};
				};
				{
					id = "COMBATSTATS_HIDE_ONOFF";
					type = K_TEXT;
					check = true;
					text = "Hide Frame";
					helptext = "When enabled the frame over the player will be hidden";
					difficulty = 2;
					feedback = function(state)
					if ( state.checked ) then
							return "Will hide.";
						else
							return "Will not hide.";
						end
					end;
					callback = function(state)
						if (state.checked) then
							CombatStats_SetHide(1);
						else
							CombatStats_SetHide(0);
						end
					end;
					default = {checked = false};
					disabled = {checked = false};
				};
			};
		}
	);
	
end
