--[[
	CS_DataConvertPre449.lua - Is a support module file for the CombatStats
	World of Warcraft addon.  This module has the functionality to update
	saved data for Pre 4.49 version CombatStats.
	
	Copyright (C) 2008 Brian Beardmore - wowaddon(at)realisticsoftware(dot)com

	Revisions:
		2007.08.26 BAB	Created by Brian Beardmore
						
	License:
		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program(see GPL.txt); if not, write to the Free Software
		Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
]]

-- Debug switch - set to true, to enable debug output for this module
local debug = false;


-------------------------------------------------------------------------------
-- Function Imports
-------------------------------------------------------------------------------
local debugPrint = CombatStats.Utils.DebugPrint;
local msgPrint = CombatStats.Utils.MsgPrint;

-------------------------------------------------------------------------------
-- Function Prototypes
-------------------------------------------------------------------------------
local importDataPre449;
local backwardSearchSpecialAttackLog;

-------------------------------------------------------------------------------
-- Data Members
-------------------------------------------------------------------------------

local _L = {
};

-------------------------------------------------------------------------------
-- Constants
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- Uses
-------------------------------------------------------------------------------
--[[ 	gCS global from CombatStats.lua file ]]



--=============================================================================
-- functions
--=============================================================================

-------------------------------------------------------------------------------
-- backwardSearchSpecialAttackLog
-------------------------------------------------------------------------------
function backwardSearchSpecialAttackLog(attackName)
	for i=gCS.specialsCount, 1, -1 do
		if ((specialAttackLog[i] ~= nil) and (specialAttackLog[i].name == attackName)) then
			return i;
		end
	end
	return 0;
end

-------------------------------------------------------------------------------
-- importDataPre449 (converts the saved data from CombatStats versions prior to 4.49
-------------------------------------------------------------------------------
function importDataPre449()
	local i;
	local index;
	local specAttck;
	local removeCount = 0;
	
	-- ONLY NEEDED PRE VERSION ?? but definitely ONLY needed for version 4.49 on
	-- in version 4.49 specialAttackLog & attackNames was removed
	
	if (gCS.dataVersion == nil or gCS.dataVersion < 449) then
		msgPrint("CombatStats is converting pre version 4.49 data . . . .");
		
		-- convert from older stored data to newer gCS;
		--	& to the overall totals into specialAttacks[CS_ATTACK_TOTAL_LABEL] record;
		if(gCS.specialsCount == nil) then
			if(specialsCount ~= nil)
				then
					gCS.specialsCount = specialsCount;
					--specialsCount = nil;
				else gCS.specialsCount = 0;
			end
		end
		if(gCS.totalDamage == nil) then
			if(totalDamage ~= nil)
				then
					gCS.totalDamage = totalDamage;
					--totalDamage = nil;
				else gCS.totalDamage = 0;
			end
		end
		
		-- verify and correct gCS.specialsCount before continuing
		--while ((specialAttackLog[gCS.specialsCount] == nil) and (gCS.specialsCount > 0)) do
		--	gCS.specialsCount = gCS.specialsCount - 1;
		--end
		removeCount = 0;
		for i=1, gCS.specialsCount do
			if (specialAttackLog[i] == nil) then
				removeCount = removeCount + 1;
			end
			if ((removeCount > 0) and (i+removeCount <= gCS.specialsCount )) then
				specialAttackLog[i] = specialAttackLog[i+removeCount];
			end
		end
		gCS.specialsCount = gCS.specialsCount - removeCount;

		if ( not specialAttacks[CS_ATTACK_TOTAL_LABEL] ) then
			specialAttacks[CS_ATTACK_TOTAL_LABEL] = {
				index				=	0,				-- has no index into the specialAttacksLog
				name				=	CS_ATTACK_TOTAL_LABEL,
				group				=	nil,
				isDOT				=	0,
				dotDmg				=	0,
				dotTicks			=	0,
				totalSwings			=	overallSwings,
				totalMisses			=	overallMisses,
				totalDodged			=	overallDodged,
				totalParried		=	overallParried,
				totalEvaded			=	overallEvaded,
				totalBlocked		=	overallBlocked,
				totalResisted		=	overallResisted,
				totalImmuned		=	overallImmuned,
				totalDeflected		=	overallDeflected,
				totalHits			=	totalHits,
				totalNonCrits		=	overallNonCrits,
				totalCrits			=	totalCrits,
				minCrit				=	overallminCrit,
				minCritTime			= 	0,
				maxCrit				=	overallmaxCrit,
				maxCritTime			= 	0,
				minReg				=	overallminReg,
				minRegTime			=	0,
				maxReg				=	overallmaxReg,
				maxRegTime			=	0,
				totalRegDmg			=	overallRegDmg,
				totalCritDmg		=	overallCritDmg,
				lastCrit			=	overallLastcrit,
				lastUpdate			=	overallLastcrit,
			};
		end
		-- Note old overallCrits and totalCrits variables should have been the same value. 
		-- totalCrits is the one being converted into the new totals storage
		
		-- verify and correct attackNames not in all the list/tables
		if (attackNames ~= nil) then
			for i=table.getn(attackNames), 1, -1 do
				if(specialAttacks[attackNames[i] ] == nil) then
					-- there is a problem because the entry is missing in the specialAttacks list
						-- see if an entry can be found in specialAttackLog
					spAttackLogIdx = backwardSearchSpecialAttackLog(attackNames[i]);
					if (spAttackLogIdx > 0)
					then
						--debugPrint("# Fixed entry for: '"..attackNames[i].."' from specialAttackLog["..spAttackLogIdx.."]");
						specialAttacks[attackNames[i] ] = specialAttackLog[spAttackLogIdx];
					else
						-- the only possible fix now is to remove the entry so the addon will continue to work
						--debugPrint("# Removing entry for: '"..attackNames[i].."'");
						attackNames[i] = nil;
					end
				end
			end
		end
		
		-- verify and correct missing values in specialAttackLog
		for i=gCS.specialsCount, 1, -1 do
			if(specialAttackLog[i].index == nil) then specialAttackLog[i].index = i; end
			--if(specialAttackLog[i].name == nil) then specialAttackLog[i].name = XX; end
			if(specialAttackLog[i].isDOT == nil) then specialAttackLog[i].isDOT = 0; end
			if(specialAttackLog[i].dotDmg == nil) then specialAttackLog[i].dotDmg = 0; end
			if(specialAttackLog[i].dotTicks == nil) then specialAttackLog[i].dotTicks = 0; end
			if(specialAttackLog[i].totalSwings == nil) then specialAttackLog[i].totalSwings = 0; end
			if(specialAttackLog[i].totalMisses == nil) then specialAttackLog[i].totalMisses = 0; end
			if(specialAttackLog[i].totalDodged == nil) then specialAttackLog[i].totalDodged = 0; end
			if(specialAttackLog[i].totalParried == nil) then specialAttackLog[i].totalParried = 0; end
			if(specialAttackLog[i].totalEvaded == nil) then specialAttackLog[i].totalEvaded = 0; end
			if(specialAttackLog[i].totalBlocked == nil) then specialAttackLog[i].totalBlocked = 0; end
			if(specialAttackLog[i].totalResisted == nil) then specialAttackLog[i].totalResisted = 0; end
			if(specialAttackLog[i].totalImmuned == nil) then specialAttackLog[i].totalImmuned = 0; end
			if(specialAttackLog[i].totalDeflected == nil) then specialAttackLog[i].totalDeflected = 0; end
			if(specialAttackLog[i].totalHits == nil) then specialAttackLog[i].totalHits = 0; end
			if(specialAttackLog[i].totalNonCrits == nil) then specialAttackLog[i].totalNonCrits = 0; end
			if(specialAttackLog[i].totalCrits == nil) then specialAttackLog[i].totalCrits = 0; end
			if(specialAttackLog[i].minCrit == nil) then specialAttackLog[i].minCrit = 0; end
			if(specialAttackLog[i].minCritTime == nil) then specialAttackLog[i].minCritTime = 0; end
			if(specialAttackLog[i].maxCrit == nil) then specialAttackLog[i].maxCrit = 0; end
			if(specialAttackLog[i].maxCritTime == nil) then specialAttackLog[i].maxCritTime = 0; end
			if(specialAttackLog[i].minReg == nil) then specialAttackLog[i].minReg = 0; end
			if(specialAttackLog[i].minRegTime == nil) then specialAttackLog[i].minRegTime = 0; end
			if(specialAttackLog[i].maxReg == nil) then specialAttackLog[i].maxReg = 0; end
			if(specialAttackLog[i].maxRegTime == nil) then specialAttackLog[i].maxRegTime = 0; end
			if(specialAttackLog[i].totalRegDmg == nil) then specialAttackLog[i].totalRegDmg = 0; end
			if(specialAttackLog[i].totalCritDmg == nil) then specialAttackLog[i].totalCritDmg = 0; end
			if(specialAttackLog[i].lastCrit == nil) then specialAttackLog[i].lastCrit = 0; end
			if(specialAttackLog[i].lastUpdate == nil) then specialAttackLog[i].lastUpdate = 0; end
			
			-- make sure all the number variables are numbers
			specialAttackLog[i].index = tonumber(specialAttackLog[i].index);
			specialAttackLog[i].dotDmg = tonumber(specialAttackLog[i].dotDmg);
			specialAttackLog[i].dotTicks = tonumber(specialAttackLog[i].dotTicks);
			specialAttackLog[i].totalSwings = tonumber(specialAttackLog[i].totalSwings);
			specialAttackLog[i].totalMisses = tonumber(specialAttackLog[i].totalMisses);
			specialAttackLog[i].totalDodged = tonumber(specialAttackLog[i].totalDodged);
			specialAttackLog[i].totalParried = tonumber(specialAttackLog[i].totalParried);
			specialAttackLog[i].totalEvaded = tonumber(specialAttackLog[i].totalEvaded);
			specialAttackLog[i].totalBlocked = tonumber(specialAttackLog[i].totalBlocked);
			specialAttackLog[i].totalResisted = tonumber(specialAttackLog[i].totalResisted);
			specialAttackLog[i].totalImmuned = tonumber(specialAttackLog[i].totalImmuned);
			specialAttackLog[i].totalDeflected = tonumber(specialAttackLog[i].totalDeflected);
			specialAttackLog[i].totalHits = tonumber(specialAttackLog[i].totalHits);
			specialAttackLog[i].totalNonCrits = tonumber(specialAttackLog[i].totalNonCrits);
			specialAttackLog[i].totalCrits = tonumber(specialAttackLog[i].totalCrits);
			specialAttackLog[i].minCrit = tonumber(specialAttackLog[i].minCrit);
			specialAttackLog[i].maxCrit = tonumber(specialAttackLog[i].maxCrit);
			specialAttackLog[i].minReg = tonumber(specialAttackLog[i].minReg);
			specialAttackLog[i].maxReg = tonumber(specialAttackLog[i].maxReg);
			specialAttackLog[i].totalRegDmg = tonumber(specialAttackLog[i].totalRegDmg);
			specialAttackLog[i].totalCritDmg = tonumber(specialAttackLog[i].totalCritDmg);
			specialAttackLog[i].lastCrit = tonumber(specialAttackLog[i].lastCrit);
			specialAttackLog[i].lastUpdate = tonumber(specialAttackLog[i].lastUpdate);
		end
	
		-- verify and correct missing values in specialAttacks
		for index, specAttck in pairs(specialAttacks) do
			-- NOTE    specialAttacks[index].name == specAtt.name
			-- need to check if this special attack is in the attackNames list
			--if ( not specialAttacks[specialName] ) then
			
			if(specAttck.index == nil) then specAttck.index = i; end
			--if(specAttck.name == nil) then specAttck.name = XX; end
			if(specAttck.isDOT == nil) then specAttck.isDOT = 0; end
			if(specAttck.dotDmg == nil) then specAttck.dotDmg = 0; end
			if(specAttck.dotTicks == nil) then specAttck.dotTicks = 0; end
			if(specAttck.totalSwings == nil) then specAttck.totalSwings = 0; end
			if(specAttck.totalMisses == nil) then specAttck.totalMisses = 0; end
			if(specAttck.totalDodged == nil) then specAttck.totalDodged = 0; end
			if(specAttck.totalParried == nil) then specAttck.totalParried = 0; end
			if(specAttck.totalEvaded == nil) then specAttck.totalEvaded = 0; end
			if(specAttck.totalBlocked == nil) then specAttck.totalBlocked = 0; end
			if(specAttck.totalResisted == nil) then specAttck.totalResisted = 0; end
			if(specAttck.totalImmuned == nil) then specAttck.totalImmuned = 0; end
			if(specAttck.totalDeflected == nil) then specAttck.totalDeflected = 0; end
			if(specAttck.totalHits == nil) then specAttck.totalHits = 0; end
			if(specAttck.totalNonCrits == nil) then specAttck.totalNonCrits = 0; end
			if(specAttck.totalCrits == nil) then specAttck.totalCrits = 0; end
			if(specAttck.minCrit == nil) then specAttck.minCrit = 0; end
			if(specAttck.minCritTime == nil) then specAttck.minCritTime = 0; end
			if(specAttck.maxCrit == nil) then specAttck.maxCrit = 0; end
			if(specAttck.maxCritTime == nil) then specAttck.maxCritTime = 0; end
			if(specAttck.minReg == nil) then specAttck.minReg = 0; end
			if(specAttck.minRegTime == nil) then specAttck.minRegTime = 0; end
			if(specAttck.maxReg == nil) then specAttck.maxReg = 0; end
			if(specAttck.maxRegTime == nil) then specAttck.maxRegTime = 0; end
			if(specAttck.totalRegDmg == nil) then specAttck.totalRegDmg = 0; end
			if(specAttck.totalCritDmg == nil) then specAttck.totalCritDmg = 0; end
			if(specAttck.lastCrit == nil) then specAttck.lastCrit = 0; end
			if(specAttck.lastUpdate == nil) then specAttck.lastUpdate = 0; end
			
			-- make sure all the number variables are numbers;
			specAttck.index = tonumber(specAttck.index);
			specAttck.dotDmg = tonumber(specAttck.dotDmg);
			specAttck.dotTicks = tonumber(specAttck.dotTicks);
			specAttck.totalSwings = tonumber(specAttck.totalSwings);
			specAttck.totalMisses = tonumber(specAttck.totalMisses);
			specAttck.totalDodged = tonumber(specAttck.totalDodged);
			specAttck.totalParried = tonumber(specAttck.totalParried);
			specAttck.totalEvaded = tonumber(specAttck.totalEvaded);
			specAttck.totalBlocked = tonumber(specAttck.totalBlocked);
			specAttck.totalResisted = tonumber(specAttck.totalResisted);
			specAttck.totalImmuned = tonumber(specAttck.totalImmuned);
			specAttck.totalDeflected = tonumber(specAttck.totalDeflected);
			specAttck.totalHits = tonumber(specAttck.totalHits);
			specAttck.totalNonCrits = tonumber(specAttck.totalNonCrits);
			specAttck.totalCrits = tonumber(specAttck.totalCrits);
			specAttck.minCrit = tonumber(specAttck.minCrit);
			specAttck.maxCrit = tonumber(specAttck.maxCrit);
			specAttck.minReg = tonumber(specAttck.minReg);
			specAttck.maxReg = tonumber(specAttck.maxReg);
			specAttck.totalRegDmg = tonumber(specAttck.totalRegDmg);
			specAttck.totalCritDmg = tonumber(specAttck.totalCritDmg);
			specAttck.lastCrit = tonumber(specAttck.lastCrit);
			specAttck.lastUpdate = tonumber(specAttck.lastUpdate);
		end
		
		specialAttackLog = nil;			-- no longer needed or used
		attackNames = nil;				-- no longer needed or used
		gCS.specialsCount = nil;		-- no longer needed or used
		
		gCS.dataVersion = 449;
	end
	
	-- note overall total missing from popup menu until the next load  (look into ??)
end


--=============================================================================
-- Initialization
--=============================================================================
if (CombatStats and CombatStats.Import and CombatStats.Import.DataPre449) then
	return;
end
debugPrint("CS_DataConvertPre449.lua loaded");

-------------------------------------------------------------------------------
-- Public API
-------------------------------------------------------------------------------
if (not CombatStats) then CombatStats = {}; end

CombatStats.Import = {
	DataPre449 = importDataPre449;
}

-------------------------------------------------------------------------------
-- Initialize before using
-------------------------------------------------------------------------------

