--[[
	CS_DetailFrame.lua - Is a support module file for the CombatStats
	World of Warcraft addon.  This modules functionality is to support all the 
	display of detailed attack information window/frame.
	
	Copyright (C) 2008 Brian Beardmore - wowaddon(at)realisticsoftware(dot)com

	Revisions:
		2007.11.23 BAB	Fixed a nil record usage bug in CombatStats_UpdateDetails
		2007.08.26 BAB	The file was created from functions extracted from the
						prior CombatStats.lua file.  The code was written by DmgInc,
						other authors and Brian Beardmore.
						
	License:
		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program(see GPL.txt); if not, write to the Free Software
		Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
]]


-- Debug switch - set to true, to enable debug output for this module
local debug = false;

-------------------------------------------------------------------------------
-- Uses and requires
-------------------------------------------------------------------------------
	-- CombatStats.Utils..  & CombatStats.AttackHistDB..

-------------------------------------------------------------------------------
-- Function Imports
-------------------------------------------------------------------------------
local sortAttackNames = CombatStats.AttackHistDB.SortAttackNames;
local setGrpSortType = CombatStats.AttackHistDB.SetGrpSortType;
local debugPrint = CombatStats.Utils.DebugPrint;

-------------------------------------------------------------------------------
-- Function Prototypes
-------------------------------------------------------------------------------
local load;

local bracketPercentText;
local isAttackHealing;

local removeAttackEntry;
local removeAttackGroup;
local removeAttackGroupCallback;
local removeAttacksOlderThan;
local removeAttacksOlderThanCallback;

local setDirtyAttackMenu;
local fillAttackNamesMenu;
local fillAttackNamesSubmenu;
local editDropDownMenuInitialize;
local groupsMenuToggleLabel;

local fillAttackNmRegCallback;
local fillAttackNmGrpCallback;
local fillAttackNmCallback;
local fillSubAttackMenuCallback;

local resetGroupList;
local clearGroupList;
local setActiveNameGroupList;
local isActiveNameGroupList;

local sortAttackMenuComp;
-------------------------------------------------------------------------------
-- Data Members
-------------------------------------------------------------------------------
local _L = {
	firstShow = 1,
	dirtyAttackMenu = 1,
	groupList = {},
};

-------------------------------------------------------------------------------
-- Constants
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- Loads this module.
-------------------------------------------------------------------------------
function load()
end

--=============================================================================
-- ?? functions
--=============================================================================

function CombatStatsAttack_OnClick()
	local attackName = this.value;
	
	if(attackName ~= nil) then
		CombatStatsAttackDropDown_SetText(attackName);
		UIDropDownMenu_SetSelectedValue(CombatStatsAttackDropDown, attackName);
		CombatStats_UpdateDetails(attackName);
	end
end

function CombatStatsAttackSubMenu_OnClick()
	local attackName = nil;
	
	if(CombatStats.AttackHistDB.GetAttackNameCount() >= this.value) then
		attackName = CombatStats.AttackHistDB.GetAttackName(this.value);
	end
	if(attackName ~= nil) then
		CombatStatsAttackDropDown_SetText(attackName);
		UIDropDownMenu_SetSelectedValue(CombatStatsAttackDropDown, attackName, 1);
		CombatStats_UpdateDetails(attackName);
	end
	--ToggleDropDownMenu(1, nil, CombatStatsAttackDropDown, nil, 1, 1);
	CloseDropDownMenus();
end

function CombatStatsAttackDropDown_GetText()
	return UIDropDownMenu_GetText(CombatStatsAttackDropDown);
end

function CombatStatsAttackDropDown_SetText(theText)
	UIDropDownMenu_SetText(theText,CombatStatsAttackDropDown);
end

function CombatStatsAttackDropDown_OnLoad()
	--debugPrint("CombatStatsAttackDropDown_OnLoad");
	
	UIDropDownMenu_Initialize(CombatStatsAttackDropDown, CombatStatsAttackDropDown_Initialize);
	--UIDropDownMenu_SetWidth(160, CombatStatsAttackDropDown)		-- now in XML file;
end

function CombatStatsText_ResetFrameGUI()
	CombatStats_Fr_GeneralNameTextLabel:SetText(CS_TEXT_NA);
	CombatStats_Fr_NonCritHitsStatText:SetText("0");
	CombatStats_Fr_NonCritDamageStatText:SetText("0");
	CombatStats_Fr_NonCritMinMaxStatText:SetText("0/0");
	CombatStats_Fr_NonCritAvgStatText:SetText(kNumberTxt);
	CombatStats_Fr_NonCritPercentDamageStatText:SetText(kPercentTxt);
	CombatStats_Fr_CritHitsStatText:SetText("0");
	CombatStats_Fr_CritDamageStatText:SetText("0");
	CombatStats_Fr_CritMinMaxStatText:SetText("0/0");
	CombatStats_Fr_CritAvgStatText:SetText(kNumberTxt);
	CombatStats_Fr_CritPercentDamageStatText:SetText(kPercentTxt);
	CombatStats_Fr_GeneralHitsSwingLineLabel:SetText(format(kHitSwingFormat,0,0));
	--CombatStats_Fr_GeneralSwingsLabel:SetText("0");
	CombatStats_Fr_GeneralMissesTextLabel:SetText("0");
	CombatStats_Fr_GeneralMissesPercentLabel:SetText(kBracketPercentTxt);
	CombatStats_Fr_GeneralDodgesTextLabel:SetText("0");
	CombatStats_Fr_GeneralDodgesPercentLabel:SetText(kBracketPercentTxt);
	CombatStats_Fr_GeneralParriedTextLabel:SetText("0");
	CombatStats_Fr_GeneralParriedPercentLabel:SetText(kBracketPercentTxt);
	CombatStats_Fr_GeneralBlockedTextLabel:SetText("0");
	CombatStats_Fr_GeneralBlockedPercentLabel:SetText(kBracketPercentTxt);
	CombatStats_Fr_GeneralResistedTextLabel:SetText("0");
	CombatStats_Fr_GeneralResistedPercentLabel:SetText(kBracketPercentTxt);
	CombatStats_Fr_GeneralImmunedTextLabel:SetText("0");
	CombatStats_Fr_GeneralImmunedPercentLabel:SetText(kBracketPercentTxt);
	CombatStats_Fr_GeneralEvadedTextLabel:SetText("0");
	CombatStats_Fr_GeneralEvadedPercentLabel:SetText(kBracketPercentTxt);
	CombatStats_Fr_GeneralDeflectedTextLabel:SetText("0");
	CombatStats_Fr_GeneralDeflectedPercentLabel:SetText(kBracketPercentTxt);
	CombatStats_Fr_GeneralPercentDmgPctLabel:SetText(kPercentTxt2);
	CombatStats_Fr_GeneralTimeLastCritTimeLabel:SetText(GREEN_FONT_COLOR_CODE..CS_TEXT_NA);
	CombatStats_Fr_OverallCritPctStatText:SetText(RED_FONT_COLOR_CODE..kPercentTxt);
	CombatStats_Fr_AttackNonCritPctStatText:SetText(GREEN_FONT_COLOR_CODE..kPercentTxt);
	CombatStats_Fr_AttackCritPctStatText:SetText(RED_FONT_COLOR_CODE..kPercentTxt);
	CombatStatsAttackDropDown_SetText(CS_DROPDOWN_SELECT_TEXT);
end

function CombatStatsText_ShowFrame()

	if(_L.firstShow == 1) then
		--
		--	Set all the default values
		--
		CombatStats_Fr_OverallCritPctLabel:SetText(CS_FRAME_GEN_ATTACK_TITLE);
		CombatStats_Fr_GeneralNameLabel:SetText(TEXT(CS_FRAME_GEN_ATTACK_NAME));
		CombatStats_Fr_GeneralMissesLabel:SetText(CS_FRAME_MISSES_TEXT);
		CombatStats_Fr_GeneralDodgesLabel:SetText(CS_FRAME_DODGES_TEXT);
		CombatStats_Fr_GeneralParriedLabel:SetText(CS_FRAME_PARRIES_TEXT);
		CombatStats_Fr_GeneralBlockedLabel:SetText(CS_FRAME_BLOCKS_TEXT);
		CombatStats_Fr_GeneralResistedLabel:SetText(CS_FRAME_RESISTS_TEXT);
		CombatStats_Fr_GeneralImmunedLabel:SetText(CS_FRAME_IMMUNE_TEXT);
		CombatStats_Fr_GeneralEvadedLabel:SetText(CS_FRAME_EVADES_TEXT );
		CombatStats_Fr_GeneralDeflectedLabel:SetText(CS_FRAME_DEFLECTS_TEXT);
		CombatStats_Fr_GeneralPercentDmgLabel:SetText(CS_FRAME_PERCENT_OVERALL_TEXT);
		CombatStats_Fr_GeneralTimeLastCritLabel:SetText(CS_FRAME_TIME_LASTCRIT_TEXT);
		CombatStats_Fr_AttackNonCritPctLabel:SetText(CS_FRAME_NONCRIT_TEXT);
		CombatStats_Fr_NonCritHitsLabel:SetText(CS_FRAME_TOTAL_TEXT);
		CombatStats_Fr_NonCritDamageLabel:SetText(CS_FRAME_DAMAGE_TEXT);
		CombatStats_Fr_NonCritMinMaxLabel:SetText(CS_FRAME_MINMAX_TEXT);
		CombatStats_Fr_NonCritAvgLabel:SetText(CS_FRAME_AVGDMG_TEXT);
		CombatStats_Fr_NonCritPercentDamageLabel:SetText(CS_FRAME_PERCENTDMG_TEXT);
		CombatStats_Fr_AttackCritPctLabel:SetText(CS_FRAME_CRIT_TEXT);
		CombatStats_Fr_CritHitsLabel:SetText(CS_FRAME_TOTAL_TEXT);
		CombatStats_Fr_CritDamageLabel:SetText(CS_FRAME_DAMAGE_TEXT);
		CombatStats_Fr_CritMinMaxLabel:SetText(CS_FRAME_MINMAX_TEXT);
		CombatStats_Fr_CritAvgLabel:SetText(CS_FRAME_AVGDMG_TEXT);
		CombatStats_Fr_CritPercentDamageLabel:SetText(CS_FRAME_PERCENTDMG_TEXT);
		
		if(gCS.currentAttack ~= nil and gCS.currentAttack ~= CS_TEXT_NA)
		then
			CombatStats_UpdateDetails(gCS.currentAttack);
			CombatStatsAttackDropDown_SetText(gCS.currentAttack);
		else
			CombatStatsText_ResetFrameGUI();
		end
		
		setDirtyAttackMenu();
		_L.firstShow = 0;
	end
	
	if( CombatStatsAttackDropDown_GetText() ~= CS_DROPDOWN_SELECT_TEXT) then
		CombatStats_UpdateDetails( UIDropDownMenu_GetText(CombatStatsAttackDropDown));
	end
	
	CombatStatsDataFrame:Show();
end

function bracketPercentText(theValue, theTotal)
	local aStr = nil;
	
	if(theTotal ~= 0) then
		if(theTotal ~= theValue) then
			aStr = format(kBracketPercentFormat, (theValue / theTotal * 100));
		else
			-- done to only show 1 '0' after decimal
			aStr = format(NORMAL_FONT_COLOR_CODE.."["..WHITE_FONT_COLOR_CODE.."%.0f%%"..NORMAL_FONT_COLOR_CODE.."]", 100);
		end
	else
		aStr = kBracketPercentTxt;
	end
	
	return aStr;
end

function isAttackHealing(attackName)
	if (attackName == nil) then return false; end
	
	return ( strsub(attackName, 0, 6) == kHealNmAType );
end
	
function CombatStats_UpdateDetails(attackName)
	local	deltaTime = 0;
	local	totAttack;
	local	recAttack;
	
	-- sanity check
	if (attackName == nil) then CombatStatsText_ResetFrameGUI() return end
	
	--
	--	Retrieve the attack records of interest
	recAttack = CombatStats.AttackHistDB.GetAttackRec(attackName, false);
	totAttack = CombatStats.AttackHistDB.GetAttackRec(CS_ATTACK_TOTAL_LABEL, false);
	if ((recAttack == nil) or (totAttack == nil)) then
		CombatStatsText_ResetFrameGUI();
		return;
	end
	
	--
	--	Setup and switch display labels if healing attack is being displayed
	gCS.currentAttack = attackName;
	if ( isAttackHealing(attackName) ) then
		CS_TT_NONCRIT_HITSPCT_TEXT = CS_TT_NONCRIT_HITSPCT_TEXT_HEAL;
		CS_TT_CRIT_HITSPCT_TEXT = CS_TT_CRIT_HITSPCT_TEXT_HEAL;
		CS_TT_OVERALLDMGPCT_TEXT = CS_TT_OVERALLDMGPCT_TEXT_HEAL;
		CS_TT_CRIT_DMGPCT_TEXT = CS_TT_CRIT_DMGPCT_TEXT_HEAL;
		CS_TT_NONCRIT_DMGPCT_TEXT = CS_TT_NONCRIT_DMGPCT_TEXT_HEAL;
		CombatStats_Fr_GeneralPercentDmgLabel:SetText(CS_FRAME_PERCENT_OVERALL_TEXT_HEAL);
		CombatStats_Fr_NonCritDamageLabel:SetText(CS_FRAME_DAMAGE_TEXT_HEAL);
		CombatStats_Fr_NonCritAvgLabel:SetText(CS_FRAME_AVGDMG_TEXT_HEAL);
		CombatStats_Fr_NonCritPercentDamageLabel:SetText(CS_FRAME_PERCENTDMG_TEXT_HEAL);
		CombatStats_Fr_CritDamageLabel:SetText(CS_FRAME_DAMAGE_TEXT_HEAL);
		CombatStats_Fr_CritAvgLabel:SetText(CS_FRAME_AVGDMG_TEXT_HEAL);
		CombatStats_Fr_CritPercentDamageLabel:SetText(CS_FRAME_PERCENTDMG_TEXT_HEAL);
	else
		CS_TT_NONCRIT_HITSPCT_TEXT = CS_TT_NONCRIT_HITSPCT_TEXT_NONHEAL;
		CS_TT_CRIT_HITSPCT_TEXT = CS_TT_CRIT_HITSPCT_TEXT_NONHEAL;
		CS_TT_OVERALLDMGPCT_TEXT = CS_TT_OVERALLDMGPCT_TEXT_NONHEAL;
		CS_TT_CRIT_DMGPCT_TEXT = CS_TT_CRIT_DMGPCT_TEXT_NONHEAL;
		CS_TT_NONCRIT_DMGPCT_TEXT = CS_TT_NONCRIT_DMGPCT_TEXT_NONHEAL;
		CombatStats_Fr_GeneralPercentDmgLabel:SetText(CS_FRAME_PERCENT_OVERALL_TEXT);
		CombatStats_Fr_NonCritDamageLabel:SetText(CS_FRAME_DAMAGE_TEXT);
		CombatStats_Fr_NonCritAvgLabel:SetText(CS_FRAME_AVGDMG_TEXT);
		CombatStats_Fr_NonCritPercentDamageLabel:SetText(CS_FRAME_PERCENTDMG_TEXT);
		CombatStats_Fr_CritDamageLabel:SetText(CS_FRAME_DAMAGE_TEXT);
		CombatStats_Fr_CritAvgLabel:SetText(CS_FRAME_AVGDMG_TEXT);
		CombatStats_Fr_CritPercentDamageLabel:SetText(CS_FRAME_PERCENTDMG_TEXT);
	end
	
	--
	-- Calculate % of hits that are crits
	--
	
	if( (totAttack.totalCrits) ~= 0) then
		CombatStats_Fr_OverallCritPctStatText:SetText( format(RED_FONT_COLOR_CODE.."%.2f %%", ( totAttack.totalCrits / totAttack.totalHits) * 100.0));
	end
	
	CombatStats_Fr_GeneralNameTextLabel:SetText(attackName);

	if ( isAttackHealing(attackName) ) then
		CombatStats_Fr_GeneralHitsSwingLineLabel:SetText(format(kHitSwingFormatHEAL,recAttack.totalHits));
	else
		CombatStats_Fr_GeneralHitsSwingLineLabel:SetText(format(kHitSwingFormat,recAttack.totalHits,recAttack.totalSwings));
	end
	
	CombatStats_Fr_GeneralMissesTextLabel:SetText(recAttack.totalMisses);
	CombatStats_Fr_GeneralMissesPercentLabel:SetText(bracketPercentText(recAttack.totalMisses, recAttack.totalSwings));
	
	CombatStats_Fr_GeneralDodgesTextLabel:SetText(recAttack.totalDodged);
	CombatStats_Fr_GeneralDodgesPercentLabel:SetText(bracketPercentText(recAttack.totalDodged, recAttack.totalSwings));
	
	CombatStats_Fr_GeneralBlockedTextLabel:SetText(recAttack.totalBlocked);
	if(attackName == CS_ATTACK_DEFENSE_LABEL) then
		CombatStats_Fr_GeneralBlockedPercentLabel:SetText(bracketPercentText(recAttack.totalBlocked, recAttack.totalHits));
	else
		CombatStats_Fr_GeneralBlockedPercentLabel:SetText(bracketPercentText(recAttack.totalBlocked, recAttack.totalSwings));
	end
	CombatStats_Fr_GeneralParriedTextLabel:SetText(recAttack.totalParried);
	CombatStats_Fr_GeneralParriedPercentLabel:SetText(bracketPercentText(recAttack.totalParried, recAttack.totalSwings));

	CombatStats_Fr_GeneralResistedTextLabel:SetText(recAttack.totalResisted);
	CombatStats_Fr_GeneralResistedPercentLabel:SetText(bracketPercentText(recAttack.totalResisted, recAttack.totalSwings));

	CombatStats_Fr_GeneralImmunedTextLabel:SetText(recAttack.totalImmuned);
	CombatStats_Fr_GeneralImmunedPercentLabel:SetText(bracketPercentText(recAttack.totalImmuned, recAttack.totalSwings));

	CombatStats_Fr_GeneralEvadedTextLabel:SetText(recAttack.totalEvaded);
	CombatStats_Fr_GeneralEvadedPercentLabel:SetText(bracketPercentText(recAttack.totalEvaded, recAttack.totalSwings));

	CombatStats_Fr_GeneralDeflectedTextLabel:SetText(recAttack.totalDeflected);
	CombatStats_Fr_GeneralDeflectedPercentLabel:SetText(bracketPercentText(recAttack.totalDeflected, recAttack.totalSwings));

	--
	--	Percent of total damage
	--
	if(attackName ~= CS_ATTACK_DEFENSE_LABEL) then
		if ( attackName == CS_ATTACK_HEAL_TOT_LABEL ) then
			CombatStats_Fr_GeneralPercentDmgPctLabel:SetText(format(kPercentFormat2,100));
		elseif ( isAttackHealing(attackName) ) then
			--[[
			FIX
			if ( recAttack.totalHits ~= 0 ) then
				CombatStats_Fr_GeneralPercentDmgPctLabel:SetText(format(kPercentFormat2, ((recAttack.totalRegDmg + recAttack.totalCritDmg + recAttack.dotDmg) / (specialAttacks[CS_ATTACK_HEAL_TOT_LABEL].totalRegDmg + specialAttacks[CS_ATTACK_HEAL_TOT_LABEL].totalCritDmg + specialAttacks[CS_ATTACK_HEAL_TOT_LABEL].dotDmg)) * 100) );
			else
				CombatStats_Fr_GeneralPercentDmgPctLabel:SetText(kPercentTxt2);
			end
			]]
			CombatStats_Fr_GeneralPercentDmgPctLabel:SetText(kPercentTxt2);
		else
			if(recAttack.totalHits ~=0) then
				CombatStats_Fr_GeneralPercentDmgPctLabel:SetText(format(kPercentFormat2, ((recAttack.totalRegDmg + recAttack.totalCritDmg + recAttack.dotDmg) / gCS.totalDamage) * 100) );
			else
				CombatStats_Fr_GeneralPercentDmgPctLabel:SetText(kPercentTxt2);
			end
		end
	end
	
	--
	--	Time since last crit
	--
	if(recAttack.lastCrit ~= 0) then
		deltaTime = time() - recAttack.lastCrit;
		if (deltaTime < (2*31526000))
		then CombatStats_Fr_GeneralTimeLastCritTimeLabel:SetText(
				GREEN_FONT_COLOR_CODE ..CombatStats.Utils.ClockFormatDeltaTime(deltaTime));
		-- fixed this time calculation to correctly use the sys date time and not uptime		--	BAB 2007.04.05;
		else CombatStats_Fr_GeneralTimeLastCritTimeLabel:SetText(GREEN_FONT_COLOR_CODE .."long ago (pre 4.0 vers)");
		end
	else
		CombatStats_Fr_GeneralTimeLastCritTimeLabel:SetText(GREEN_FONT_COLOR_CODE..CS_TEXT_NA);
	end
	
	--
	--	Attack Crit %
	--
	if (recAttack.totalHits ~= 0) and (recAttack.totalCrits ~= 0) then
		CombatStats_Fr_AttackCritPctStatText:SetText(format(RED_FONT_COLOR_CODE..kPercentFormat, ( (recAttack.totalCrits) / (recAttack.totalHits)) * 100.0 ));
	else
		CombatStats_Fr_AttackCritPctStatText:SetText(RED_FONT_COLOR_CODE..kPercentTxt);
	end
	
	--
	--	Attack Non Crit %
	--
	if (recAttack.totalHits ~= 0) and (recAttack.totalNonCrits ~= 0) then
		CombatStats_Fr_AttackNonCritPctStatText:SetText( format(GREEN_FONT_COLOR_CODE..kPercentFormat, ( (recAttack.totalNonCrits) / (recAttack.totalHits)) * 100.0 ));
	else
		CombatStats_Fr_AttackNonCritPctStatText:SetText(GREEN_FONT_COLOR_CODE..kPercentTxt);
	end
	
	--
	--	Non Crit stats
	--
	CombatStats_Fr_NonCritHitsStatText:SetText(recAttack.totalNonCrits);
	CombatStats_Fr_NonCritDamageStatText:SetText(recAttack.totalRegDmg);
	CombatStats_Fr_NonCritMinMaxStatText:SetText(recAttack.minReg .."/"..recAttack.maxReg);
	if( recAttack.totalRegDmg ~= 0) then
		if( (recAttack.totalHits - recAttack.totalCrits) < 0) then
			CombatStats_Fr_NonCritAvgStatText:SetText(format(kNumberFormat,	recAttack.totalRegDmg / (recAttack.totalCrits - recAttack.totalHits ) ));
		elseif( (recAttack.totalHits - recAttack.totalCrits) == 0 ) then
			CombatStats_Fr_NonCritAvgStatText:SetText(format(kNumberFormat, recAttack.totalRegDmg / (recAttack.totalCrits) ));
		elseif( (recAttack.totalHits - recAttack.totalCrits) > 0 ) then
			CombatStats_Fr_NonCritAvgStatText:SetText(format(kNumberFormat,recAttack.totalRegDmg / (recAttack.totalHits - recAttack.totalCrits) ));
		end
	else
		CombatStats_Fr_NonCritAvgStatText:SetText(kNumberTxt);
	end
	if(recAttack.totalNonCrits ~=0) then
		CombatStats_Fr_NonCritPercentDamageStatText:SetText(format(kPercentFormat2, ( recAttack.totalRegDmg	  / (recAttack.totalRegDmg + recAttack.totalCritDmg )) * 100.0));
	else
		CombatStats_Fr_NonCritPercentDamageStatText:SetText(kPercentTxt2);
	end
	
	--
	--	Crit stats
	--
	CombatStats_Fr_CritHitsStatText:SetText(recAttack.totalCrits);
	CombatStats_Fr_CritDamageStatText:SetText(recAttack.totalCritDmg);
	CombatStats_Fr_CritMinMaxStatText:SetText(recAttack.minCrit .."/"..recAttack.maxCrit);
	if( recAttack.totalCritDmg ~= 0) then
		CombatStats_Fr_CritAvgStatText:SetText(format(kNumberFormat, (recAttack.totalCritDmg / recAttack.totalCrits) ));
	else
		CombatStats_Fr_CritAvgStatText:SetText(kNumberTxt);
	end
	if(recAttack.totalCrits ~=0) then
		CombatStats_Fr_CritPercentDamageStatText:SetText(format(kPercentFormat2, (recAttack.totalCritDmg / ( recAttack.totalRegDmg + recAttack.totalCritDmg )) * 100.0 ));
	else
		CombatStats_Fr_CritPercentDamageStatText:SetText(kPercentTxt2);
	end
	
	--
	--	Show DOT Stats
	--
	if (recAttack.isDOT ==1) then
		CombatStats_Fr_GeneralHitsSwingLineLabel:SetText(format(kHitSwingFormatDOT,recAttack.dotTicks));
		CombatStats_Fr_NonCritHitsStatText:SetText(recAttack.dotTicks);
		CombatStats_Fr_NonCritDamageStatText:SetText(recAttack.dotDmg);
		CombatStats_Fr_NonCritMinMaxStatText:SetText(recAttack.minReg .."/"..recAttack.maxReg);
		CombatStats_Fr_NonCritAvgStatText:SetText(format(kNumberFormat,  recAttack.dotDmg /  recAttack.dotTicks	));
		CombatStats_Fr_NonCritPercentDamageStatText:SetText("100.0 %");
		CombatStats_Fr_AttackNonCritPctStatText:SetText(GREEN_FONT_COLOR_CODE.."100.0%");
		
		--
		--	Should never be a 0 tick entry but just in case
		--
			--		if(specialAttacks[attackNames[y]].dotTicks ~= 0) then
			--			text = text.."\n"..format(TEXT(DOT_AVERAGE),(specialAttacks[attackNames[y]].dotDmg / specialAttacks[attackNames[y]].dotTicks));
			--		end
	elseif ( recAttack.isDOT ==2) then
		CombatStats_Fr_GeneralHitsSwingLineLabel:SetText(format(kHitSwingFormatDOT2,recAttack.dotTicks));
		CombatStats_Fr_NonCritHitsStatText:SetText(recAttack.dotTicks);
		CombatStats_Fr_NonCritDamageStatText:SetText(recAttack.dotDmg);
		CombatStats_Fr_NonCritMinMaxStatText:SetText(recAttack.minReg .."/"..recAttack.maxReg);
		CombatStats_Fr_NonCritAvgStatText:SetText(format(kNumberFormat,  recAttack.dotDmg /  recAttack.dotTicks	));
		CombatStats_Fr_NonCritPercentDamageStatText:SetText("100.0 %");
		CombatStats_Fr_AttackNonCritPctStatText:SetText(GREEN_FONT_COLOR_CODE.."100.0%");
	end

end

--
-- Attack data entry modifying functions
--

function sortAttackMenuComp(a,b)
	--debugPrint("a: "..a.." - b: "..b);
	local aRec = CombatStats.AttackHistDB.GetAttackRec(a, false);
	local bRec = CombatStats.AttackHistDB.GetAttackRec(b, false);
	if ((aRec == nil) or (bRec == nil)) then return false; end
	--debugPrint("CombatStats_Config.CombatStats_AttackMenuSort = "..CombatStats_Config.CombatStats_AttackMenuSort);
	
	if(CombatStats_Config.CombatStats_AttackMenuSort == 3) then
		-- sort order, largest max non-crit;
		if (aRec.maxReg == bRec.maxReg) then
			return a<b;
		end
		return aRec.maxReg>bRec.maxReg;
	elseif(CombatStats_Config.CombatStats_AttackMenuSort == 2) then
		-- sort order, largest max crit;
		if (aRec.maxCrit == bRec.maxCrit) then
			return a<b;
		end
		return aRec.maxCrit>bRec.maxCrit;
	elseif(CombatStats_Config.CombatStats_AttackMenuSort == 1) then
		-- sort order, newest last crit;
		if (aRec.lastCrit == bRec.lastCrit) then
			return a<b;
		end
		return aRec.lastCrit>bRec.lastCrit;
	else
		-- sort order, attack name;
		return a<b;
	end
end

function removeAttackEntry(attackName)
	if (attackName == nil) then return; end

	CombatStats.AttackHistDB.ClearRecFromAttacksDB(attackName);
	clearGroupList(true);		--- needs more work
	
	if(attackName == gCS.currentAttack) then
		gCS.currentAttack = CS_TEXT_NA;
		CombatStatsText_ResetFrameGUI();
	end
end

function removeAttackGroupCallback(attackName, groupName)
	local grpNm = CombatStats.AttackHistDB.GetGroupNameForAttackName(attackName,CombatStats_Config.CombatStats_AttackMenu1stGrping);
	if ((grpNm ~= nil) and (attackName ~= nil)) then
		debugPrint("  grpNm: "..grpNm..", for attackName: "..attackName);
	end
	
	if ((grpNm ~= nil) and (grpNm == groupName)) then
		debugPrint("Attack: "..attackName..", in group: "..groupName..", is being deleted");
		removeAttackEntry(attackName);
	end
	
end

function removeAttackGroup(groupName)
	if (groupName == nil) then return; end
	
	CombatStats.AttackHistDB.CallFunctionForEachAttackNameRev(removeAttackGroupCallback, groupName);
end

function removeAttacksOlderThanCallback(attackName, olderDaysToDelete)
	local recAttack = CombatStats.AttackHistDB.GetAttackRec(attackName,false);
	
	if ((recAttack ~= nil) and (recAttack.lastCrit ~= nil)) then
		--debugPrint("recAttack.lastCrit: "..recAttack.lastCrit);
		if (olderDaysToDelete ~= 0)
		then
			--debugPrint("(time() - recAttack.lastCrit)/(24*60*60) = "..(time() - recAttack.lastCrit)/(24*60*60));
			if ((time() - recAttack.lastCrit)/(24*60*60) > olderDaysToDelete ) then
				removeAttackEntry(attackName);
			end
		else
			-- recAttack.lastCrit is known to be 0 at this point;
			--debugPrint("olderDaysToDelete: "..olderDaysToDelete);
			if (recAttack.lastCrit == 0) then
				removeAttackEntry(attackName);
			end
		end
	end
	
end

function removeAttacksOlderThan(olderDaysToDelete)
	if (olderDaysToDelete == nil) then return; end
	
	CombatStats.AttackHistDB.CallFunctionForEachAttackNameRev(removeAttacksOlderThanCallback, olderDaysToDelete);
end

--
-- Attack Main Drop down menu functionality
--

function CombatStatsAttackDropDown_Initialize(level)
	--debugPrint("CombatStatsAttackDropDown_Initialize");
	level = level or 1;
	if (level == 1) then
		--[[
		if ( table.getn(groupNames) == 0 ) then
			CombatStats.AttackHistDB.SortAttackNames(sortAttackMenuComp);
			CombatStats_RebuildGroupList();
		end
		]]
		fillAttackNamesMenu();
	end
	if (level == 2) then
		--debugPrint("UIDROPDOWNMENU_MENU_VALUE : ^^"..UIDROPDOWNMENU_MENU_VALUE.."^^");
		fillAttackNamesSubmenu(UIDROPDOWNMENU_MENU_VALUE);
	end
end

function CombatStatsAttackDropDown_OnClick()
	--debugPrint("CombatStatsAttackDropDown_OnClick");
	ToggleDropDownMenu(1, nil, CombatStatsAttackDropDown);	-- default;
end

function setDirtyAttackMenu()
	_L.dirtyAttackMenu = 1;
end

function fillAttackNmRegCallback(attackName)
	-- add dropdown		-- FIX
	local recAttack = CombatStats.AttackHistDB.GetAttackRec(attackName,false);
	if ((recAttack ~= nil) and (recAttack.group == nil)) then
		CombatStats.Utils.AppendMenuItem(attackName, attackName, CombatStatsAttack_OnClick, dropdown);
	end
end

function fillAttackNmGrpCallback(attackName)
	setActiveNameGroupList(CombatStats.AttackHistDB.GetGroupNameForAttackName(attackName,CombatStats_Config.CombatStats_AttackMenu1stGrping));
end

function fillAttackNmCallback(attackName,dropdown)
	local grpNm = CombatStats.AttackHistDB.GetGroupNameForAttackName(attackName,CombatStats_Config.CombatStats_AttackMenu1stGrping);
	if (grpNm == nil) then
		CombatStats.Utils.AppendMenuItem(attackName, attackName, CombatStatsAttack_OnClick, dropdown);
	else
		setActiveNameGroupList(CombatStats.AttackHistDB.GetGroupNameForAttackName(attackName,CombatStats_Config.CombatStats_AttackMenu1stGrping));
	end
end

function fillSubAttackMenuCallback(attackName,groupName,dropdown)
	local grpNm = CombatStats.AttackHistDB.GetGroupNameForAttackName(attackName,CombatStats_Config.CombatStats_AttackMenu1stGrping);
	if ((grpNm ~= nil) and (grpNm == groupName)) then
		CombatStats.Utils.AppendMenuItemSubmenuItem(
			attackName, 
			CombatStats.AttackHistDB.GetAttackNameIdx(attackName),
			CombatStatsAttackSubMenu_OnClick, dropdown	);
	end
end


function resetGroupList()
	local i;
	
	for i, _ in pairs(_L.groupList) do
		_L.groupList[i] = false;
	end
end

function clearGroupList(justInactive)
	local i;
	local value;
	justInactive = justInactive or false;
	
	if (justInactive) then
		for i, value in pairs(_L.groupList) do
			value = nil;
		end
	else
		_L.groupList = {}
	end
end

function setActiveNameGroupList(groupName)
	_L.groupList[groupName] = true;
end

function isActiveNameGroupList(groupName)
	if (groupName == nil) then return false; end
	
	return((_L.groupList[groupName] ~= nil) and (_L.groupList[groupName] == true));
end

function fillAttackNamesMenu()
	--debugPrint("fillAttackNamesMenu");
	local dropdown = this:GetParent();
	local iGrpNm;
	local value;
	
	if (_L.dirtyAttackMenu == 1) then
		--debugPrint("fillAttackNamesMenu--SORT");
		sortAttackNames(sortAttackMenuComp);
		_L.dirtyAttackMenu = 0;
	end
	
	resetGroupList();
	CombatStats.AttackHistDB.CallFunctionForEachAttackName(fillAttackNmCallback, dropdown);
	for iGrpNm, value in pairs(_L.groupList) do
		if (value == true) then
			CombatStats.Utils.AppendMenuItemSubmenu(iGrpNm, iGrpNm, nil, dropdown);
		end
		--[[
		if (value == true) then
			debugPrint("iGrpNm/value "..iGrpNm.."/true");
		else
			debugPrint("iGrpNm/value "..iGrpNm.."/false");
		end
		]]
	end
	
end

function fillAttackNamesSubmenu(groupName)
	local dropdown = this:GetParent();
	CombatStats.AttackHistDB.CallFunctionForEachAttackName(fillSubAttackMenuCallback, groupName, dropdown);
end

--
-- Edit Drop down menu functionality
--

function CombatStatsEditDropDown_OnLoad()
	UIDropDownMenu_Initialize(CombatStatsEditDropDown, editDropDownMenuInitialize);
	--UIDropDownMenu_SetWidth(10, CombatStatsEditDropDown)		-- in XML file now;
end

function editDropDownMenuInitialize(level)
	local info;
	local aStr;
	local dropdown = this:GetParent();
	local currentAttack = nil;
	local attackGroup = nil;
	
	level = level or 1;
	
	if (level == 1) then
		if (CombatStats_Config.CombatStats_AttackMenuSort == 0)
			then CombatStats.Utils.AppendMenuChkItem(CS_ED_DROPDN_AZSORT_TEXT,0,CombatStatsEditDropDown_ChangeAttackSort,dropdown);
			else CombatStats.Utils.AppendMenuItem(CS_ED_DROPDN_AZSORT_TEXT,0,CombatStatsEditDropDown_ChangeAttackSort,dropdown);
		end
		if (CombatStats_Config.CombatStats_AttackMenuSort == 1)
			then CombatStats.Utils.AppendMenuChkItem(CS_ED_DROPDN_TIMESORT_TEXT,1,CombatStatsEditDropDown_ChangeAttackSort,dropdown);
			else CombatStats.Utils.AppendMenuItem(CS_ED_DROPDN_TIMESORT_TEXT,1,CombatStatsEditDropDown_ChangeAttackSort,dropdown);
		end
		if (CombatStats_Config.CombatStats_AttackMenuSort == 2)
			then CombatStats.Utils.AppendMenuChkItem(CS_ED_DROPDN_MXCRITSORT_TEXT,2,CombatStatsEditDropDown_ChangeAttackSort,dropdown);
			else CombatStats.Utils.AppendMenuItem(CS_ED_DROPDN_MXCRITSORT_TEXT,2,CombatStatsEditDropDown_ChangeAttackSort,dropdown);
		end
		if (CombatStats_Config.CombatStats_AttackMenuSort == 3)
			then CombatStats.Utils.AppendMenuChkItem(CS_ED_DROPDN_MXNONCRITSORT_TEXT,3,CombatStatsEditDropDown_ChangeAttackSort,dropdown);
			else CombatStats.Utils.AppendMenuItem(CS_ED_DROPDN_MXNONCRITSORT_TEXT,3,CombatStatsEditDropDown_ChangeAttackSort,dropdown);
		end
		CombatStats.Utils.AppendMenuItemSeparator("- - - - -",dropdown);
		CombatStats.Utils.AppendMenuItem(groupsMenuToggleLabel(),1,CombatStatsEditDropDown_ToggleBeginGroups,dropdown);
		currentAttack = gCS.currentAttack;
		
		if(currentAttack ~= nil and currentAttack ~= CS_TEXT_NA and currentAttack ~= CS_ATTACK_TOTAL_LABEL
						and currentAttack ~= CS_ATTACK_TOTAL_LABEL) then
			CombatStats.Utils.AppendMenuItemSeparator("- - - - -",dropdown);
			if(string.len(currentAttack) < 16)
				then aStr = string.format(CS_ED_DROPDN_DEL__DAYSATTACK_TEXT, currentAttack);
				else aStr = string.format(CS_ED_DROPDN_DEL__DAYSATTACK_TEXT, string.sub(currentAttack,1,15).."...");
			end
			CombatStats.Utils.AppendMenuItem(aStr,currentAttack,CombatStatsEditDropDown_DeleteAnAttack,dropdown);
			attackGroup = CombatStats.AttackHistDB.GetGroupNameForAttackName(currentAttack);
			if(attackGroup ~= nil) then
				if(string.len(attackGroup) < 16)
					then aStr = string.format(CS_ED_DROPDN_DEL__DAYSATTACKGRP_TEXT, attackGroup);
					else aStr = string.format(CS_ED_DROPDN_DEL__DAYSATTACKGRP_TEXT, string.sub(attackGroup,1,15).."...");
				end
				CombatStats.Utils.AppendMenuItem(aStr,attackGroup,CombatStatsEditDropDown_DeleteAttackGroup,dropdown);
			end
		end
		
		CombatStats.Utils.AppendMenuItemSeparator("- - - - -",dropdown);
		CombatStats.Utils.AppendMenuItem(CS_ED_DROPDN_DEL_1_DAYS_TEXT,1,CombatStatsEditDropDown_DeleteAttacksOlderThan,dropdown);
		CombatStats.Utils.AppendMenuItem(CS_ED_DROPDN_DEL_5_DAYS_TEXT,5,CombatStatsEditDropDown_DeleteAttacksOlderThan,dropdown);
		CombatStats.Utils.AppendMenuItem(CS_ED_DROPDN_DEL_7_DAYS_TEXT,7,CombatStatsEditDropDown_DeleteAttacksOlderThan,dropdown);
		CombatStats.Utils.AppendMenuItem(CS_ED_DROPDN_DEL_30_DAYS_TEXT,30,CombatStatsEditDropDown_DeleteAttacksOlderThan,dropdown);
		CombatStats.Utils.AppendMenuItem(CS_ED_DROPDN_DEL_90_DAYS_TEXT,90,CombatStatsEditDropDown_DeleteAttacksOlderThan,dropdown);
		CombatStats.Utils.AppendMenuItem(CS_ED_DROPDN_DEL_180_DAYS_TEXT,180,CombatStatsEditDropDown_DeleteAttacksOlderThan,dropdown);
		CombatStats.Utils.AppendMenuItem(CS_ED_DROPDN_DEL_365_DAYS_TEXT,365,CombatStatsEditDropDown_DeleteAttacksOlderThan,dropdown);
		CombatStats.Utils.AppendMenuItem(CS_ED_DROPDN_DEL_730_DAYS_TEXT,730,CombatStatsEditDropDown_DeleteAttacksOlderThan,dropdown);
		CombatStats.Utils.AppendMenuItem(CS_ED_DROPDN_DEL_0_DAYS_TEXT,0,CombatStatsEditDropDown_DeleteAttacksOlderThan,dropdown);
	end
end

function CombatStatsEditDropDown_OnClick()
	--debugPrint("CombatStatsAttackDropDown_OnClick");
	ToggleDropDownMenu(1, nil, CombatStatsEditDropDown);	-- default;
end

function groupsMenuToggleLabel()
	if(CombatStats_Config.CombatStats_AttackMenu1stGrping == 1)
		then return CS_ED_DROPDN_DISABLE_GROUPING_TEXT;
		else return CS_ED_DROPDN_ENABLE_GROUPING_TEXT;
	end
end

function CombatStatsEditDropDown_ToggleBeginGroups()
	--debugPrint("CombatStats_Config.CombatStats_AttackMenu1stGrping "..TEXT(CombatStats_Config.CombatStats_AttackMenu1stGrping));
	if(CombatStats_Config.CombatStats_AttackMenu1stGrping == 0)
		then CombatStats_Config.CombatStats_AttackMenu1stGrping = 1;
		else CombatStats_Config.CombatStats_AttackMenu1stGrping = 0;
	end
	setGrpSortType(CombatStats_Config.CombatStats_AttackMenu1stGrping);
	
	setDirtyAttackMenu();
end

function CombatStatsEditDropDown_ChangeAttackSort()
	--debugPrint("attackMenuSort before: "..CombatStats_Config.CombatStats_AttackMenuSort..", after: "..this.value);
	CombatStats_Config.CombatStats_AttackMenuSort = this.value;
	setDirtyAttackMenu();
end

function CombatStatsEditDropDown_DeleteAnAttack()
	removeAttackEntry(this.value);
end

function CombatStatsEditDropDown_DeleteAttackGroup()
	removeAttackGroup(this.value);
end

function CombatStatsEditDropDown_DeleteAttacksOlderThan()
	removeAttacksOlderThan(this.value);
end

--=============================================================================
-- Initialization
--=============================================================================
if (CombatStats and CombatStats.DetailFrame) then return; end;
debugPrint("CS_DetailFrame.lua loaded");

-------------------------------------------------------------------------------
-- Public API
-------------------------------------------------------------------------------
if (not CombatStats) then CombatStats = {}; end

CombatStats.DetailFrame = {
	Load = load;
	
	SetDirtyAttackMenu = setDirtyAttackMenu;
}

--[[
CombatStatsAttack_OnClick
CombatStatsAttackSubMenu_OnClick
CombatStatsAttackDropDown_GetText
CombatStatsAttackDropDown_SetText
CombatStatsAttackDropDown_OnLoad
CombatStatsText_ResetFrameGUI
CombatStatsText_ShowFrame
CombatStats_UpdateDetails
CombatStats_AddToGrpList
CombatStatsAttackDropDown_Initialize
CombatStatsAttackDropDown_OnClick
CombatStatsEditDropDown_OnLoad
CombatStatsEditDropDown_OnClick
CombatStatsEditDropDown_ToggleBeginGroups
CombatStatsEditDropDown_ChangeAttackSort
CombatStatsEditDropDown_DeleteAnAttack
CombatStatsEditDropDown_DeleteAttackGroup
CombatStatsEditDropDown_DeleteAttacksOlderThan
]]
