--[[
	CS_Utils.lua - Is a support module file for the CombatStats
	World of Warcraft addon.  This module has utility functions for CombatStats.
	
	Copyright (C) 2008 Brian Beardmore - wowaddon(at)realisticsoftware(dot)com

	Revisions:
		2007.09.19 BAB	Created by Brian Beardmore.
						
	License:
		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program(see GPL.txt); if not, write to the Free Software
		Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
]]

-- Debug switch - set to true, to enable debug output for this module
local debug = true;

-------------------------------------------------------------------------------
-- Uses and requires
-------------------------------------------------------------------------------
	-- CombatStats.Utils..

-------------------------------------------------------------------------------
-- Function Imports
-------------------------------------------------------------------------------
local debugPrint = CombatStats.Utils.DebugPrint;

-------------------------------------------------------------------------------
-- Function Prototypes
-------------------------------------------------------------------------------
local chkSet_UpPSValue;

-------------------------------------------------------------------------------
-- Data Members
-------------------------------------------------------------------------------

--local _L = {
--	err_Only_Numbers = false;
--};

-------------------------------------------------------------------------------
-- Constants
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Uses
-------------------------------------------------------------------------------


--=============================================================================
-- Static popup dialog functions & Global Setup
--=============================================================================

-------------------------------------------------------------------------------
-- chkSet_UpPSValue
-------------------------------------------------------------------------------
--function CombatStats_ChkSet_UpPSValue(theText)
function chkSet_UpPSValue(theText)
	if ((theText ~= nil) and (CombatStats.Utils.IsTextNumbers(theText))) then
		-- got only numbers from the input text
		CombatStats_UpdateFramePer(tonumber(theText));
		return true;
	end
	
	return false;
end

-------------------------------------------------------------------------------
-- StaticPopupDialogs["CombatStats_GUI_Get_Max_UpdatePreSec"]
-------------------------------------------------------------------------------
StaticPopupDialogs["CombatStats_GUI_Get_Max_UpdatePreSec"] = {
	text = "Set CombatStats maximum updates-per-second value",
	button1 = ACCEPT,
	button2 = CANCEL,
	hasEditBox = 1,
	maxLetters = 5,
	OnAccept = function()
		if (StaticPopup_Visible("CombatStats_GUI_Err_Only_Numbers")) then
			StaticPopup_Hide("CombatStats_GUI_Err_Only_Numbers");
			return true;
		end
		
		local editBox = getglobal(this:GetParent():GetName().."EditBox");
		local editText = editBox:GetText();
		
		if (CombatStats.Dialogs.ChkSet_UpPSValue(editText)) then
			return false;		-- return false on success
		end
		
		-- bad input value so display an error and continue
		--cs_print("Err: only number values can be entered");
		StaticPopup_Show("CombatStats_GUI_Err_Only_Numbers");
		editText = string.gsub(editText, "[^0-9]", "")		-- erase all non-numbers
		editBox:SetText(editText);							-- update the input number
		return true;		-- to continue and to not hide
	end,
	OnShow = function()
		getglobal(this:GetName().."EditBox"):SetFocus();
	end,
	OnHide = function()
		if ( ChatFrameEditBox:IsShown() ) then
			ChatFrameEditBox:SetFocus();
		end
		getglobal(this:GetName().."EditBox"):SetText("");
	end,
	EditBoxOnEnterPressed = function()
		if (StaticPopup_Visible("CombatStats_GUI_Err_Only_Numbers")) then
			StaticPopup_Hide("CombatStats_GUI_Err_Only_Numbers");
			return;
		end
		
		local editBox = getglobal(this:GetParent():GetName().."EditBox");
		local editText = editBox:GetText();
		
		if (CombatStats.Dialogs.ChkSet_UpPSValue(editText)) then
			this:GetParent():Hide();	-- hide on success
			return;		
		end
		
		-- bad input value so display an error and continue
		--cs_print("Err: only number values can be entered");
		StaticPopup_Show("CombatStats_GUI_Err_Only_Numbers");
		editText = string.gsub(editText, "[^0-9]", "")		-- erase all non-numbers
		editBox:SetText(editText);							-- update the input number
	end,
	EditBoxOnEscapePressed = function()
		this:GetParent():Hide();
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
};

-------------------------------------------------------------------------------
-- StaticPopupDialogs["CombatStats_GUI_Err_Only_Numbers"]
-------------------------------------------------------------------------------
StaticPopupDialogs["CombatStats_GUI_Err_Only_Numbers"] = {
	text = "Only numbers can be entered",
	button1 = OKAY,
	timeout = 0,
	whileDead = 1,
	showAlert = 1,
};

--=============================================================================
-- Initialization
--=============================================================================
if (CombatStats and CombatStats.Dialogs) then return end;
debugPrint("CS_Dialogs.lua loaded");

-------------------------------------------------------------------------------
-- Public API
-------------------------------------------------------------------------------
if (not CombatStats) then CombatStats = {}; end

CombatStats.Dialogs = {
	ChkSet_UpPSValue = chkSet_UpPSValue,
};

